type token =
  | CHOICE
  | STAR
  | COMMA
  | LPAREN
  | RPAREN
  | LBRACKET
  | RBRACKET
  | BAR
  | SEMI
  | NEW
  | OUT
  | IN
  | AT
  | IDENT of (Pitptree.ident)
  | TAG of (Pitptree.ident)
  | STRING of (Pitptree.ident)
  | PROJECTION of (Pitptree.ident)
  | INT of (int)
  | REPL
  | IF
  | THEN
  | ELSE
  | EQUAL
  | FUN
  | EQUATION
  | REDUCTION
  | PREDICATE
  | PROCESS
  | SLASH
  | DOT
  | EOF
  | LET
  | QUERY
  | BEFORE
  | PUTBEGIN
  | NONINTERF
  | EVENT
  | NOT
  | ELIMTRUE
  | FREE
  | SUCHTHAT
  | CLAUSES
  | RED
  | EQUIV
  | EQUIVEQ
  | WEDGE
  | DIFF
  | COLON
  | NOUNIF
  | PHASE
  | BARRIER
  | AMONG
  | WEAKSECRET
  | PARAM
  | ORTEXT
  | FAIL
  | LESS
  | GREATER
  | GEQ
  | PLUS
  | MINUS
  | TYPE
  | SET
  | FORALL
  | CONST
  | INJEVENT
  | OR
  | CHANNEL
  | LETFUN
  | DEFINE
  | EXPAND
  | YIELD
  | LEQ
  | PROBA
  | LBRACE
  | RBRACE
  | PROOF
  | IMPLEMENTATION
  | EQUIVALENCE
  | OTHERWISE
  | FOREACH
  | DO
  | SECRET
  | PUBLICVARS
  | RANDOM
  | LEFTARROW
  | LEMMA
  | AXIOM
  | RESTRICTION
  | FOR
  | TABLE
  | INSERT
  | GET

open Parsing;;
let _ = parse_error;;
# 2 "pitparser.mly"
(*************************************************************
 *                                                           *
 *  Cryptographic protocol verifier                          *
 *                                                           *
 *  Bruno Blanchet, Vincent Cheval, and Marc Sylvestre       *
 *                                                           *
 *  Copyright (C) INRIA, CNRS 2000-2020                      *
 *                                                           *
 *************************************************************)

(*

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details (in file LICENSE).

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)
# 31 "pitparser.mly"

open Parsing_helper
open Ptree
open Pitptree

let zero () = PIdent (("0",parse_extent())), parse_extent()

let rec unfold_int t = function
  | 0 -> t
  | n -> PFunApp(("+", parse_extent()), [unfold_int t (n-1)]), parse_extent()

 let rec unfold_int_minus t = function
   | 0 -> t
   | n -> PFunApp(("- "^(string_of_int n), parse_extent()), [t]), parse_extent()

let gzero () = PGIdent (("0",parse_extent())), parse_extent()

let rec unfold_gint t = function
  | 0 -> t
  | n -> PGFunApp(("+", parse_extent()), [unfold_gint t (n-1)], None), parse_extent()

let pzero () = PPIdent (("0",parse_extent())), parse_extent()

let rec unfold_pint t = function
  | 0 -> t
  | n -> PPFunApp(("+", parse_extent()), [unfold_pint t (n-1)]), parse_extent()

let rec unfold_pint_minus t = function
  | 0 -> t
  | n -> PPFunApp(("- "^(string_of_int n), parse_extent()), [t]), parse_extent()

let pat_zero () = PPatFunApp (("0",parse_extent()), [])

let rec unfold_pat_int t = function
  | 0 -> t
  | n -> PPatFunApp(("+", parse_extent()), [unfold_pat_int t (n-1)])

exception Syntax

# 167 "pitparser.ml"
let yytransl_const = [|
  257 (* CHOICE *);
  258 (* STAR *);
  259 (* COMMA *);
  260 (* LPAREN *);
  261 (* RPAREN *);
  262 (* LBRACKET *);
  263 (* RBRACKET *);
  264 (* BAR *);
  265 (* SEMI *);
  266 (* NEW *);
  267 (* OUT *);
  268 (* IN *);
  269 (* AT *);
  275 (* REPL *);
  276 (* IF *);
  277 (* THEN *);
  278 (* ELSE *);
  279 (* EQUAL *);
  280 (* FUN *);
  281 (* EQUATION *);
  282 (* REDUCTION *);
  283 (* PREDICATE *);
  284 (* PROCESS *);
  285 (* SLASH *);
  286 (* DOT *);
    0 (* EOF *);
  287 (* LET *);
  288 (* QUERY *);
  289 (* BEFORE *);
  290 (* PUTBEGIN *);
  291 (* NONINTERF *);
  292 (* EVENT *);
  293 (* NOT *);
  294 (* ELIMTRUE *);
  295 (* FREE *);
  296 (* SUCHTHAT *);
  297 (* CLAUSES *);
  298 (* RED *);
  299 (* EQUIV *);
  300 (* EQUIVEQ *);
  301 (* WEDGE *);
  302 (* DIFF *);
  303 (* COLON *);
  304 (* NOUNIF *);
  305 (* PHASE *);
  306 (* BARRIER *);
  307 (* AMONG *);
  308 (* WEAKSECRET *);
  309 (* PARAM *);
  310 (* ORTEXT *);
  311 (* FAIL *);
  312 (* LESS *);
  313 (* GREATER *);
  314 (* GEQ *);
  315 (* PLUS *);
  316 (* MINUS *);
  317 (* TYPE *);
  318 (* SET *);
  319 (* FORALL *);
  320 (* CONST *);
  321 (* INJEVENT *);
  322 (* OR *);
  323 (* CHANNEL *);
  324 (* LETFUN *);
  325 (* DEFINE *);
  326 (* EXPAND *);
  327 (* YIELD *);
  328 (* LEQ *);
  329 (* PROBA *);
  330 (* LBRACE *);
  331 (* RBRACE *);
  332 (* PROOF *);
  333 (* IMPLEMENTATION *);
  334 (* EQUIVALENCE *);
  335 (* OTHERWISE *);
  336 (* FOREACH *);
  337 (* DO *);
  338 (* SECRET *);
  339 (* PUBLICVARS *);
  340 (* RANDOM *);
  341 (* LEFTARROW *);
  342 (* LEMMA *);
  343 (* AXIOM *);
  344 (* RESTRICTION *);
  345 (* FOR *);
  346 (* TABLE *);
  347 (* INSERT *);
  348 (* GET *);
    0|]

let yytransl_block = [|
  270 (* IDENT *);
  271 (* TAG *);
  272 (* STRING *);
  273 (* PROJECTION *);
  274 (* INT *);
    0|]

let yylhs = "\255\255\
\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\002\000\025\000\025\000\025\000\001\000\001\000\027\000\027\000\
\027\000\027\000\027\000\027\000\027\000\027\000\027\000\027\000\
\027\000\028\000\028\000\021\000\021\000\022\000\022\000\029\000\
\029\000\029\000\029\000\030\000\030\000\033\000\033\000\034\000\
\034\000\035\000\035\000\031\000\031\000\036\000\037\000\037\000\
\032\000\032\000\006\000\006\000\039\000\039\000\039\000\038\000\
\038\000\011\000\011\000\040\000\040\000\040\000\041\000\041\000\
\016\000\016\000\042\000\042\000\008\000\008\000\007\000\007\000\
\043\000\043\000\005\000\005\000\005\000\005\000\005\000\005\000\
\005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
\005\000\045\000\045\000\044\000\044\000\019\000\019\000\019\000\
\019\000\046\000\046\000\046\000\026\000\026\000\018\000\018\000\
\048\000\048\000\048\000\048\000\047\000\047\000\049\000\049\000\
\020\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
\020\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
\020\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
\020\000\052\000\052\000\050\000\050\000\053\000\053\000\053\000\
\053\000\051\000\051\000\017\000\017\000\017\000\017\000\017\000\
\056\000\056\000\057\000\057\000\054\000\054\000\054\000\054\000\
\054\000\054\000\054\000\054\000\059\000\059\000\055\000\055\000\
\060\000\060\000\060\000\060\000\058\000\058\000\061\000\061\000\
\062\000\023\000\023\000\014\000\014\000\063\000\063\000\064\000\
\064\000\009\000\009\000\010\000\010\000\012\000\012\000\065\000\
\065\000\065\000\065\000\024\000\024\000\066\000\066\000\067\000\
\067\000\068\000\068\000\069\000\070\000\070\000\071\000\071\000\
\013\000\013\000\013\000\013\000\013\000\013\000\013\000\013\000\
\013\000\013\000\013\000\013\000\013\000\013\000\013\000\013\000\
\013\000\013\000\013\000\013\000\013\000\013\000\013\000\013\000\
\013\000\013\000\013\000\073\000\073\000\079\000\079\000\074\000\
\074\000\076\000\076\000\075\000\075\000\075\000\075\000\075\000\
\075\000\075\000\075\000\075\000\080\000\080\000\077\000\077\000\
\015\000\015\000\015\000\015\000\015\000\015\000\015\000\015\000\
\015\000\015\000\015\000\015\000\015\000\015\000\015\000\015\000\
\015\000\015\000\015\000\015\000\015\000\015\000\015\000\015\000\
\015\000\015\000\081\000\081\000\078\000\078\000\082\000\082\000\
\083\000\083\000\072\000\072\000\084\000\084\000\003\000\003\000\
\003\000\004\000\004\000\000\000\000\000\000\000\000\000\000\000"

let yylen = "\002\000\
\005\000\010\000\012\000\005\000\007\000\005\000\004\000\007\000\
\008\000\005\000\007\000\006\000\009\000\006\000\009\000\006\000\
\006\000\006\000\007\000\005\000\007\000\005\000\006\000\004\000\
\004\000\006\000\004\000\005\000\004\000\005\000\004\000\006\000\
\004\000\004\000\007\000\003\000\009\000\007\000\007\000\005\000\
\000\000\001\000\001\000\001\000\004\000\005\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\002\000\001\000\003\000\001\000\003\000\005\000\
\005\000\004\000\004\000\001\000\001\000\001\000\003\000\003\000\
\003\000\001\000\003\000\003\000\000\000\003\000\001\000\003\000\
\003\000\000\000\003\000\000\000\001\000\003\000\005\000\003\000\
\001\000\003\000\001\000\000\000\002\000\003\000\005\000\003\000\
\003\000\001\000\003\000\000\000\001\000\001\000\001\000\000\000\
\003\000\001\000\001\000\004\000\004\000\006\000\001\000\001\000\
\003\000\003\000\003\000\003\000\003\000\004\000\003\000\003\000\
\003\000\003\000\001\000\001\000\000\000\007\000\005\000\003\000\
\001\000\000\000\005\000\009\000\004\000\002\000\003\000\001\000\
\002\000\004\000\004\000\004\000\000\000\002\000\000\000\002\000\
\005\000\001\000\001\000\003\000\003\000\003\000\003\000\003\000\
\003\000\007\000\007\000\003\000\003\000\004\000\003\000\003\000\
\006\000\005\000\005\000\005\000\003\000\003\000\005\000\002\000\
\006\000\003\000\001\000\001\000\000\000\006\000\004\000\005\000\
\003\000\001\000\000\000\006\000\006\000\002\000\006\000\006\000\
\002\000\000\000\002\000\000\000\004\000\006\000\001\000\003\000\
\005\000\002\000\002\000\006\000\003\000\001\000\001\000\000\000\
\006\000\004\000\005\000\003\000\001\000\000\000\002\000\000\000\
\004\000\003\000\001\000\001\000\000\000\003\000\000\000\006\000\
\005\000\005\000\004\000\006\000\004\000\002\000\004\000\003\000\
\001\000\003\000\003\000\004\000\003\000\003\000\000\000\001\000\
\003\000\003\000\003\000\003\000\001\000\000\000\000\000\007\000\
\002\000\003\000\002\000\005\000\002\000\005\000\006\000\001\000\
\001\000\006\000\004\000\005\000\008\000\008\000\004\000\007\000\
\004\000\005\000\008\000\006\000\009\000\003\000\007\000\004\000\
\003\000\003\000\006\000\002\000\000\000\002\000\000\000\002\000\
\000\000\001\000\003\000\001\000\003\000\004\000\006\000\008\000\
\001\000\003\000\003\000\002\000\003\000\001\000\001\000\000\000\
\004\000\006\000\001\000\001\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\003\000\003\000\004\000\003\000\003\000\
\007\000\005\000\005\000\007\000\005\000\007\000\006\000\007\000\
\010\000\003\000\002\000\000\000\002\000\000\000\003\000\000\000\
\003\000\001\000\001\000\000\000\003\000\003\000\000\000\001\000\
\003\000\003\000\001\000\002\000\002\000\002\000\002\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\042\000\
\043\000\044\000\000\000\076\001\000\000\000\000\077\001\000\000\
\078\001\000\000\000\000\079\001\000\000\000\000\000\000\000\000\
\000\000\000\000\107\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\124\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\172\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\137\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\036\000\000\000\000\000\000\000\000\000\182\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\050\000\055\000\
\056\000\057\000\047\000\048\000\049\000\054\000\051\000\052\000\
\053\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\248\000\000\000\000\000\000\000\000\000\000\000\000\000\249\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\070\001\073\001\074\001\000\000\
\000\000\121\000\000\000\000\000\115\000\000\000\000\000\000\000\
\000\000\114\000\113\000\000\000\101\000\102\000\000\000\000\000\
\103\000\099\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\212\000\000\000\000\000\000\000\
\166\000\000\000\000\000\000\000\096\000\149\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\148\000\
\000\000\000\000\142\000\097\000\135\000\000\000\128\000\000\000\
\000\000\000\000\024\000\000\000\007\000\000\000\027\000\090\000\
\033\000\000\000\000\000\210\000\000\000\214\000\000\000\000\000\
\000\000\000\000\229\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\199\000\187\000\000\000\000\000\000\000\
\000\000\000\000\000\000\025\000\000\000\000\000\000\000\000\000\
\000\000\000\000\034\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\029\000\000\000\059\000\000\000\000\000\
\000\000\000\000\000\000\031\000\063\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\243\000\
\000\000\245\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\020\001\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\045\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\122\000\108\000\109\000\118\000\000\000\
\000\000\000\000\083\000\000\000\006\000\223\000\004\000\000\000\
\000\000\010\000\000\000\000\000\000\000\170\000\000\000\000\000\
\000\000\178\000\000\000\000\000\000\000\139\000\140\000\000\000\
\158\000\000\000\138\000\000\000\000\000\022\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\209\000\000\000\000\000\
\000\000\000\000\000\000\228\000\000\000\195\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\020\000\028\000\001\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\067\001\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\030\000\061\000\000\000\
\069\000\068\000\000\000\067\000\066\000\000\000\242\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\019\001\000\000\236\000\000\000\000\000\000\000\000\000\000\000\
\000\000\031\001\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\009\001\000\000\010\001\000\000\
\000\000\000\000\000\000\000\000\046\000\000\000\000\000\000\000\
\133\000\040\000\000\000\000\000\105\000\086\000\000\000\088\000\
\000\000\000\000\000\000\012\000\000\000\000\000\000\000\167\000\
\095\000\000\000\000\000\145\000\000\000\162\000\163\000\000\000\
\164\000\000\000\000\000\023\000\000\000\026\000\207\000\032\000\
\000\000\000\000\192\000\000\000\000\000\000\000\197\000\000\000\
\000\000\000\000\000\000\000\000\000\000\016\000\017\000\018\000\
\000\000\000\000\000\000\000\000\058\001\000\000\000\000\000\000\
\039\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\014\000\000\000\000\000\000\000\000\000\000\000\
\038\001\037\001\000\000\000\000\000\000\000\000\000\000\000\000\
\065\000\000\000\064\000\000\000\093\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\230\000\000\000\251\000\
\000\000\000\000\000\000\000\000\021\001\000\000\000\000\027\001\
\000\000\000\000\026\001\000\000\008\001\000\000\000\000\000\000\
\000\000\000\000\001\001\000\000\000\000\000\000\110\000\000\000\
\000\000\220\000\000\000\000\000\161\000\000\000\000\000\144\000\
\000\000\000\000\000\000\021\000\000\000\008\000\035\000\000\000\
\000\000\000\000\000\000\205\000\189\000\000\000\185\000\181\000\
\180\000\184\000\183\000\019\000\005\000\000\000\000\000\065\001\
\000\000\033\001\000\000\000\000\000\000\000\000\000\000\000\000\
\046\001\000\000\000\000\000\000\000\000\038\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\011\000\094\000\000\000\
\000\000\000\000\000\000\244\000\235\000\234\000\000\000\233\000\
\246\000\000\000\252\000\000\000\029\001\022\001\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\039\000\000\000\131\000\
\000\000\000\000\087\000\009\000\000\000\000\000\000\000\154\000\
\155\000\126\000\000\000\000\000\000\000\193\000\000\000\000\000\
\000\000\000\000\000\000\000\000\051\001\000\000\000\000\063\001\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\081\000\
\000\000\000\000\000\000\076\000\250\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\011\001\247\000\004\001\
\000\000\000\000\000\000\000\000\000\000\000\000\013\000\176\000\
\000\000\190\000\000\000\000\000\196\000\015\000\034\001\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\037\000\078\000\
\080\000\000\000\072\000\073\000\075\000\237\000\000\000\000\000\
\000\000\000\000\000\000\000\001\007\001\000\000\000\000\000\000\
\000\000\000\000\002\000\174\000\000\000\000\000\000\000\054\001\
\052\001\000\000\000\000\000\000\254\000\253\000\240\000\000\000\
\003\001\000\000\000\000\000\000\000\000\000\000\203\000\000\000\
\000\000\071\000\024\001\000\000\005\001\132\000\003\000\000\000\
\201\000\000\000\000\000\218\000\057\001\000\000\000\000\000\000\
\000\000\216\000"

let yydgoto = "\006\000\
\036\000\037\000\041\000\044\000\127\000\144\000\031\001\032\001\
\076\003\057\000\086\000\055\000\004\001\044\001\002\002\073\000\
\099\000\074\000\079\001\154\000\234\000\115\000\045\001\092\000\
\038\000\122\000\235\000\236\000\116\000\035\002\163\002\161\002\
\099\003\252\002\253\002\248\002\249\002\036\001\037\001\041\002\
\076\000\056\000\033\001\128\000\129\000\010\001\183\000\077\000\
\100\002\155\000\209\001\156\000\210\001\106\001\107\001\121\002\
\206\000\219\002\108\001\220\002\230\001\088\000\093\000\148\003\
\203\000\159\001\047\002\048\002\005\001\103\003\160\001\003\002\
\069\002\011\003\056\002\137\001\057\002\074\003\131\003\058\002\
\045\003\142\002\004\002\042\000"

let yysindex = "\132\004\
\103\013\103\013\053\255\065\255\112\001\000\000\208\255\097\255\
\097\255\238\255\029\000\085\002\063\000\090\000\083\255\175\001\
\101\000\105\255\131\255\110\000\101\000\121\000\139\000\101\000\
\101\000\173\000\189\000\035\001\041\001\220\255\094\255\000\000\
\000\000\000\000\063\001\000\000\126\255\083\255\000\000\211\000\
\000\000\255\000\222\000\000\000\106\001\112\001\116\001\148\001\
\095\001\155\001\000\000\108\004\170\001\166\001\209\001\112\001\
\209\001\112\001\011\001\093\000\214\001\010\001\217\001\001\000\
\167\001\223\001\179\255\237\001\245\001\249\001\243\001\255\001\
\253\001\209\001\015\005\008\002\016\002\199\255\024\002\011\002\
\183\255\035\002\191\004\141\000\233\001\003\002\037\002\050\002\
\055\002\023\002\101\000\103\013\112\001\023\001\064\002\081\002\
\088\002\087\002\209\001\070\002\209\001\209\001\071\002\051\002\
\072\002\203\000\100\002\111\002\090\002\060\255\103\002\104\002\
\109\002\122\002\108\002\130\002\137\002\009\011\009\011\134\002\
\150\001\209\001\129\002\053\255\065\255\112\001\115\000\141\002\
\000\000\112\001\112\001\112\001\112\001\112\001\112\001\112\001\
\133\002\145\002\112\001\129\255\045\255\143\002\152\002\144\002\
\199\000\153\002\199\004\129\255\154\002\101\000\009\011\010\001\
\178\002\015\011\180\002\000\000\183\002\101\000\010\001\129\255\
\010\001\164\002\146\002\151\002\010\001\010\001\010\001\118\002\
\010\001\035\003\172\002\010\001\010\001\010\001\010\001\010\001\
\010\001\010\001\187\002\010\001\010\001\101\000\000\000\166\001\
\035\003\189\002\207\002\198\002\103\013\129\255\103\013\010\001\
\103\013\101\000\103\013\129\255\112\001\101\000\129\255\204\002\
\000\000\084\003\236\255\147\003\196\002\000\000\194\002\147\003\
\147\003\150\255\190\002\103\013\193\002\195\002\069\001\129\255\
\103\013\101\000\020\002\129\255\129\255\103\013\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\149\002\060\255\210\002\205\002\206\002\208\002\209\002\
\103\013\094\255\129\255\009\011\219\002\230\002\232\002\085\255\
\000\000\072\011\020\002\168\001\242\002\239\002\247\002\000\000\
\252\002\253\002\255\002\014\000\009\011\199\002\242\010\010\001\
\213\002\007\003\248\002\211\000\000\000\000\000\000\000\118\000\
\112\001\000\000\019\003\106\000\000\000\159\003\250\000\033\001\
\228\000\000\000\000\000\108\004\000\000\000\000\022\003\029\003\
\000\000\000\000\012\003\030\003\040\003\103\013\097\255\103\013\
\112\001\033\003\103\013\041\003\000\000\080\000\227\012\010\001\
\000\000\063\255\004\003\047\003\000\000\000\000\010\001\101\000\
\101\000\049\003\068\002\052\003\209\001\176\003\209\001\103\013\
\212\001\176\010\181\002\025\002\006\003\015\001\143\001\000\000\
\176\010\120\002\000\000\000\000\000\000\174\255\000\000\112\001\
\204\255\037\003\000\000\058\003\000\000\144\011\000\000\000\000\
\000\000\016\003\249\003\000\000\209\001\000\000\112\001\112\001\
\112\001\105\255\000\000\067\003\073\003\147\003\076\003\087\003\
\085\003\095\003\096\003\000\000\000\000\147\003\097\003\098\003\
\054\000\209\001\103\013\000\000\103\013\103\013\078\003\079\003\
\081\003\209\001\000\000\101\003\107\003\020\002\108\003\019\255\
\082\003\020\002\168\001\124\003\138\003\133\003\139\003\048\014\
\070\003\151\003\153\003\000\000\103\013\000\000\060\255\136\003\
\207\000\143\003\144\003\000\000\000\000\157\003\101\001\161\003\
\020\002\020\002\020\002\139\255\129\255\129\255\089\003\000\000\
\052\255\000\000\234\013\183\003\031\002\003\000\132\003\020\002\
\154\003\194\255\000\000\084\001\188\003\169\001\128\003\198\003\
\204\003\009\011\000\000\205\003\020\002\016\000\209\001\183\001\
\010\001\103\013\112\001\000\000\000\000\000\000\000\000\169\003\
\129\255\022\255\000\000\152\002\000\000\000\000\000\000\035\000\
\209\001\000\000\194\003\103\013\010\001\000\000\197\003\206\003\
\220\003\000\000\129\255\254\255\105\005\000\000\000\000\199\003\
\000\000\199\003\000\000\010\000\200\003\000\000\189\002\223\003\
\198\002\103\013\208\003\103\013\174\003\000\000\103\013\209\003\
\108\004\108\004\108\004\000\000\147\003\000\000\235\003\237\003\
\147\003\218\003\147\003\219\003\234\003\219\003\219\003\217\003\
\000\000\000\000\000\000\103\013\103\013\103\013\224\003\232\003\
\020\002\232\012\010\004\000\000\161\003\020\002\129\255\020\002\
\007\014\241\003\004\000\022\004\020\002\023\004\024\004\020\002\
\103\013\020\002\020\002\020\002\020\002\020\002\013\004\014\004\
\020\002\020\002\020\002\215\003\004\004\000\000\000\000\030\004\
\000\000\000\000\031\004\000\000\000\000\011\004\000\000\254\000\
\251\003\251\012\023\013\038\004\068\001\000\004\039\004\052\004\
\000\000\188\003\000\000\042\004\009\011\031\002\066\255\058\255\
\053\004\000\000\031\002\129\255\031\002\115\007\020\002\020\002\
\043\004\020\002\188\003\009\011\000\000\046\004\000\000\055\004\
\020\002\031\002\205\003\196\004\000\000\027\004\058\004\101\000\
\000\000\000\000\158\001\129\255\000\000\000\000\101\000\000\000\
\097\255\036\004\009\011\000\000\017\004\010\001\054\004\000\000\
\000\000\064\004\062\004\000\000\010\001\000\000\000\000\063\004\
\000\000\103\013\079\004\000\000\103\013\000\000\000\000\000\000\
\103\013\081\004\000\000\240\255\080\004\147\003\000\000\068\004\
\059\004\150\255\059\004\059\004\103\013\000\000\000\000\000\000\
\103\013\020\002\041\013\020\002\000\000\067\004\097\004\107\004\
\000\000\020\002\020\002\020\002\020\002\161\003\122\005\020\002\
\031\002\038\001\000\000\157\007\076\001\216\001\191\001\154\001\
\000\000\000\000\089\014\193\003\138\013\103\013\103\013\092\004\
\000\000\161\255\000\000\103\013\000\000\110\004\129\255\020\002\
\031\002\109\004\104\004\113\004\118\004\000\000\122\004\000\000\
\009\011\219\000\064\255\031\002\000\000\116\004\000\000\000\000\
\002\003\188\013\000\000\117\004\000\000\205\003\130\004\060\004\
\135\004\137\004\000\000\103\013\118\002\048\004\000\000\144\255\
\069\004\000\000\103\013\165\000\000\000\020\005\010\001\000\000\
\199\003\094\014\199\003\000\000\198\002\000\000\000\000\147\003\
\120\004\127\004\139\004\000\000\000\000\136\004\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\065\014\020\002\000\000\
\129\255\000\000\020\002\024\014\193\013\217\013\142\004\140\004\
\000\000\145\004\151\004\020\002\082\004\000\000\138\004\149\004\
\152\004\147\004\148\004\164\004\168\004\000\000\000\000\188\003\
\065\013\142\255\126\004\000\000\000\000\000\000\163\004\000\000\
\000\000\009\011\000\000\031\002\000\000\000\000\167\004\009\011\
\161\003\188\003\009\011\188\003\153\004\000\000\176\004\000\000\
\105\255\166\004\000\000\000\000\103\013\063\255\169\013\000\000\
\000\000\000\000\190\004\147\003\181\004\000\000\147\003\103\013\
\106\004\197\004\089\014\020\002\000\000\020\002\020\002\000\000\
\020\002\198\004\153\004\089\014\103\013\192\004\092\004\000\000\
\193\004\193\004\161\255\000\000\000\000\143\004\209\001\201\004\
\205\003\078\255\009\011\219\000\188\003\000\000\000\000\000\000\
\020\002\209\001\203\004\209\001\112\001\103\013\000\000\000\000\
\063\255\000\000\204\004\147\003\000\000\000\000\000\000\020\002\
\089\014\024\014\024\014\089\014\020\002\209\001\000\000\000\000\
\000\000\209\004\000\000\000\000\000\000\000\000\188\003\188\003\
\213\004\179\004\219\000\000\000\000\000\089\014\211\004\220\004\
\202\004\024\005\000\000\000\000\240\255\219\004\089\014\000\000\
\000\000\089\014\221\004\193\004\000\000\000\000\000\000\129\255\
\000\000\009\011\212\004\156\004\103\013\112\001\000\000\240\255\
\020\002\000\000\000\000\205\003\000\000\000\000\000\000\067\000\
\000\000\024\014\105\255\000\000\000\000\112\001\215\007\112\001\
\067\000\000\000"

let yyrindex = "\000\000\
\133\255\235\005\238\005\000\000\000\000\000\000\000\000\189\003\
\189\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\050\004\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\239\005\240\005\000\000\000\000\000\005\165\010\000\000\
\170\010\000\000\000\000\008\006\000\000\000\000\236\004\000\000\
\236\004\000\000\236\004\000\000\000\000\006\005\000\000\187\012\
\206\011\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\236\004\180\000\154\255\078\000\239\004\000\000\000\000\
\000\000\000\000\000\000\067\007\000\000\000\000\000\000\229\000\
\173\255\000\000\000\000\024\000\000\000\119\000\000\000\000\000\
\000\000\000\000\236\004\000\000\236\004\236\004\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\240\004\000\000\000\000\000\000\000\000\
\188\000\236\004\000\000\238\005\000\000\000\000\022\005\000\000\
\000\000\000\005\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\025\005\000\000\000\000\000\000\000\000\
\127\000\000\000\000\000\025\005\000\000\026\005\000\000\000\000\
\090\011\032\005\000\000\000\000\235\011\000\000\006\005\000\000\
\000\000\000\000\000\000\000\000\006\005\000\000\006\005\180\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\024\000\025\005\024\000\000\000\
\024\000\000\000\024\000\000\000\000\000\000\000\000\000\000\000\
\000\000\171\000\000\000\034\005\000\000\000\000\000\000\034\005\
\034\005\000\000\000\000\024\000\000\000\000\000\000\000\000\000\
\024\000\026\005\000\000\025\005\025\005\024\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\023\255\223\004\000\000\000\000\000\000\000\000\
\024\000\000\000\025\005\000\000\000\000\000\000\000\000\057\002\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\184\000\000\000\009\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\098\001\125\000\
\206\000\000\000\000\000\026\000\000\000\000\000\000\000\035\005\
\000\000\000\000\111\001\000\000\039\005\024\000\189\003\024\000\
\000\000\000\000\024\000\000\000\000\000\000\000\000\000\000\000\
\000\000\042\005\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\187\000\000\000\236\004\024\000\
\182\012\073\000\056\001\153\012\037\012\066\012\095\012\000\000\
\185\002\124\012\000\000\000\000\000\000\230\000\000\000\000\000\
\239\004\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\055\001\000\000\000\000\236\004\000\000\000\000\000\000\
\000\000\050\004\000\000\000\000\000\000\034\005\000\000\014\003\
\000\000\046\005\000\000\000\000\000\000\000\000\000\000\000\000\
\119\000\236\004\024\000\000\000\024\000\024\000\000\000\000\000\
\000\000\236\004\000\000\000\000\000\000\047\005\000\000\150\002\
\240\002\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\024\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\007\005\
\000\000\000\000\047\005\000\000\000\000\000\000\000\000\000\000\
\057\002\000\000\000\000\000\000\050\005\037\000\114\001\000\000\
\000\000\000\000\000\000\012\008\059\008\059\008\000\000\000\000\
\000\000\000\000\000\000\201\008\000\000\000\000\236\004\000\000\
\000\000\024\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\241\255\
\236\004\000\000\000\000\024\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\148\011\000\000\000\000\000\000\148\011\
\000\000\148\011\000\000\148\011\000\000\000\000\000\000\000\000\
\000\000\024\000\000\000\024\000\000\000\000\000\024\000\000\000\
\191\000\210\000\036\001\000\000\000\000\000\000\000\000\121\004\
\034\005\000\000\000\000\043\000\000\000\043\000\043\000\000\000\
\000\000\000\000\000\000\024\000\024\000\024\000\000\000\000\000\
\000\000\051\005\000\000\000\000\007\005\047\005\000\000\000\000\
\000\000\000\000\000\000\178\001\000\000\000\000\000\000\000\000\
\024\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\039\001\
\000\000\000\000\042\001\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\055\005\
\000\000\059\008\000\000\000\000\000\000\000\000\205\255\052\005\
\000\000\000\000\050\005\000\000\000\000\218\001\000\000\000\000\
\000\000\047\005\059\008\000\000\000\000\000\000\000\000\000\000\
\047\005\050\005\248\008\059\008\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\189\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\024\000\028\005\000\000\024\000\000\000\000\000\000\000\
\024\000\000\000\000\000\060\005\000\000\000\000\000\000\000\000\
\119\000\000\000\119\000\119\000\024\000\000\000\000\000\000\000\
\024\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\047\005\059\005\000\000\047\005\
\050\005\014\005\000\000\104\005\180\004\074\003\165\003\255\003\
\000\000\000\000\195\005\089\004\000\000\013\005\024\000\000\000\
\000\000\000\000\000\000\024\000\000\000\000\000\000\000\000\000\
\000\000\106\008\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\112\010\000\000\000\000\000\000\000\000\112\255\000\000\
\154\008\039\009\000\000\000\000\000\000\086\009\000\000\000\000\
\000\000\000\000\000\000\024\000\069\005\000\000\000\000\236\004\
\000\000\000\000\024\000\000\000\000\000\075\005\000\000\000\000\
\148\011\109\003\148\011\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\025\007\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\084\005\
\000\000\000\000\000\000\086\005\000\000\000\000\000\000\059\008\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\133\009\000\000\
\224\001\059\008\000\000\059\008\073\007\000\000\000\000\000\000\
\050\004\000\000\000\000\000\000\024\000\000\000\088\005\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\024\000\
\000\000\000\000\243\005\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\251\000\041\006\024\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\127\007\174\007\000\000\
\180\009\000\000\000\000\112\010\059\008\000\000\000\000\000\000\
\000\000\154\008\000\000\236\004\000\000\024\000\000\000\000\000\
\000\000\000\000\089\005\000\000\000\000\000\000\000\000\000\000\
\089\006\025\007\025\007\137\006\000\000\085\005\000\000\000\000\
\000\000\137\001\000\000\000\000\000\000\000\000\059\008\059\008\
\000\000\034\000\112\010\000\000\000\000\222\007\227\009\000\000\
\000\000\000\000\000\000\000\000\000\000\092\005\185\006\000\000\
\000\000\233\006\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\018\010\000\000\024\000\000\000\000\000\000\000\
\000\000\000\000\000\000\065\010\000\000\000\000\000\000\245\255\
\000\000\025\007\050\004\000\000\000\000\000\000\000\000\000\000\
\056\000\000\000"

let yygindex = "\000\000\
\000\000\002\000\232\005\233\005\251\255\209\255\146\255\099\255\
\000\000\012\004\083\000\064\005\152\255\142\005\017\000\006\000\
\054\255\157\255\244\255\057\000\232\004\134\005\018\000\023\005\
\000\000\014\255\000\000\150\005\000\000\000\000\000\000\000\000\
\244\252\000\000\071\003\000\000\077\003\206\004\000\000\097\254\
\000\000\249\255\194\004\003\006\026\255\000\000\093\255\000\000\
\073\254\239\000\000\000\091\005\119\253\163\254\076\255\118\001\
\009\001\000\000\154\004\099\254\000\000\000\000\251\252\245\002\
\000\000\000\000\225\003\000\000\000\000\000\000\222\003\132\254\
\089\254\206\252\010\255\178\255\227\253\092\003\000\000\228\003\
\215\252\000\000\015\004\025\006"

let yytablesize = 4006
let yytable = "\052\000\
\080\000\058\000\053\001\039\000\061\001\170\001\071\002\114\001\
\069\001\146\000\085\000\149\000\067\002\179\001\007\001\077\002\
\245\001\108\003\079\000\077\003\082\000\183\001\006\002\041\000\
\098\000\119\000\171\000\111\001\112\001\182\002\044\002\058\000\
\102\002\087\000\103\002\086\002\105\002\042\001\090\001\006\001\
\006\001\093\001\188\001\120\000\194\002\100\003\046\001\158\000\
\120\003\121\003\145\000\211\000\147\000\213\000\214\000\155\001\
\129\003\156\001\122\001\142\000\180\002\223\000\224\000\225\000\
\226\000\157\001\012\003\040\000\075\000\059\002\063\001\083\000\
\006\001\227\000\011\001\228\000\207\001\229\000\043\000\084\001\
\141\003\208\001\230\000\061\000\106\003\077\001\062\000\202\000\
\155\001\231\000\156\001\160\000\063\000\201\000\121\000\087\002\
\064\000\058\000\157\001\090\000\065\000\134\002\007\002\101\000\
\149\003\224\001\104\000\105\000\200\000\138\001\139\001\138\003\
\157\001\066\000\096\000\243\002\065\002\111\000\068\000\069\000\
\016\001\232\000\065\002\052\002\120\000\020\001\021\001\022\001\
\023\001\024\001\025\001\157\001\150\001\028\001\019\001\158\001\
\065\002\135\002\176\002\151\001\011\002\150\003\029\001\114\002\
\094\000\162\001\063\003\070\000\080\003\143\000\233\000\096\000\
\045\002\118\000\112\000\189\002\180\001\113\000\182\001\054\000\
\041\000\095\000\098\000\113\001\195\002\006\001\096\000\091\000\
\158\001\025\003\019\001\006\001\072\000\171\001\250\002\082\001\
\223\001\239\001\091\000\091\000\095\000\091\000\006\001\114\000\
\006\001\096\000\190\000\251\002\046\002\076\001\083\001\091\001\
\085\001\098\000\087\001\030\001\089\001\188\002\081\002\116\003\
\065\002\186\000\091\000\119\000\193\002\117\000\225\001\018\001\
\047\001\018\001\041\000\018\001\191\000\116\001\163\000\092\001\
\064\002\054\001\123\001\091\000\097\000\053\000\059\001\140\001\
\187\000\062\001\075\000\018\001\065\001\066\001\067\001\068\001\
\069\001\070\001\071\001\136\001\073\001\074\001\240\002\097\000\
\051\001\075\000\148\001\164\000\098\001\160\000\221\000\091\000\
\086\001\187\000\219\000\059\000\065\002\217\002\187\000\049\002\
\050\002\169\001\218\002\158\000\159\000\158\000\059\002\018\001\
\075\001\099\001\098\002\163\001\051\001\219\001\221\000\221\001\
\239\002\069\001\219\000\242\002\088\001\178\001\098\002\178\001\
\222\002\032\003\140\002\033\003\119\000\116\000\119\000\119\000\
\119\000\116\000\119\000\200\001\023\001\110\000\023\001\197\001\
\023\001\199\001\060\000\089\002\202\001\232\001\099\002\160\000\
\186\000\060\002\116\000\041\000\171\001\097\002\116\000\119\000\
\023\001\204\000\104\002\018\001\117\002\217\000\065\002\179\002\
\121\000\222\001\248\001\119\000\119\000\119\000\184\002\186\000\
\186\000\075\002\255\001\165\000\078\000\165\000\165\000\165\000\
\136\000\165\000\205\000\136\000\165\000\217\000\171\001\178\001\
\061\003\233\001\234\001\235\001\023\001\137\000\138\000\018\001\
\150\000\116\000\041\000\006\001\116\000\041\000\165\000\081\000\
\119\000\136\002\070\003\136\000\072\003\204\001\190\001\213\001\
\136\000\069\003\089\000\151\000\249\001\017\001\250\001\251\001\
\187\001\116\000\035\003\100\000\188\000\137\000\138\000\120\000\
\134\000\120\000\120\000\120\000\222\000\120\000\102\000\078\002\
\010\002\134\000\214\001\215\001\134\000\052\001\030\002\194\000\
\130\000\147\003\009\002\058\001\188\000\060\001\135\000\136\000\
\103\000\090\002\120\000\165\000\222\000\109\003\183\002\135\000\
\136\000\165\000\135\000\136\000\137\000\138\000\120\000\120\000\
\120\000\042\002\043\002\139\000\178\001\137\000\138\000\225\002\
\137\000\138\000\002\003\225\000\139\000\083\002\200\002\139\000\
\062\002\141\000\106\000\082\002\141\000\134\000\120\000\125\003\
\126\003\130\000\029\003\084\000\130\000\076\002\083\003\224\000\
\225\000\085\003\107\000\120\000\178\002\092\002\218\000\039\001\
\117\000\141\000\117\000\117\000\117\000\134\000\117\000\135\003\
\084\000\130\000\226\000\190\002\224\000\134\000\033\002\092\000\
\034\002\219\000\178\001\108\002\117\000\110\002\006\001\171\001\
\112\002\211\000\145\003\117\000\171\001\211\000\171\001\226\000\
\010\003\121\000\204\002\135\000\136\000\006\001\118\003\117\000\
\117\000\117\000\117\000\171\001\123\000\126\002\127\002\128\002\
\062\001\137\000\138\000\129\000\165\002\093\002\062\001\124\000\
\139\000\000\003\061\000\089\000\006\001\062\000\148\000\117\000\
\143\000\131\002\147\002\063\000\091\000\066\003\125\000\153\000\
\137\002\158\000\204\000\065\000\117\000\143\002\137\000\138\000\
\146\002\023\003\148\002\149\002\150\002\151\002\152\002\136\000\
\066\000\155\002\156\002\157\002\227\000\068\000\069\000\082\000\
\108\000\088\001\077\000\205\000\137\000\138\000\109\000\134\000\
\239\000\208\000\160\000\208\000\160\000\160\000\160\000\208\000\
\160\000\227\000\171\001\160\000\082\000\160\000\176\000\077\000\
\009\003\179\000\070\000\042\003\117\000\135\000\136\000\185\002\
\186\002\058\000\119\001\019\002\120\001\160\000\121\001\066\002\
\160\000\040\002\171\001\137\000\138\000\020\002\021\002\022\002\
\023\002\024\002\006\001\072\000\116\000\171\001\116\000\116\000\
\116\000\039\002\116\000\212\002\178\001\026\002\214\002\126\000\
\045\000\085\000\215\002\046\000\025\001\085\000\025\001\130\000\
\025\001\160\000\166\002\171\002\172\002\047\000\228\002\116\000\
\048\000\049\000\229\002\020\002\021\002\022\002\023\002\024\002\
\025\001\015\003\160\000\116\000\116\000\116\000\116\000\070\000\
\160\000\070\000\230\002\026\002\050\000\035\001\206\002\131\000\
\026\003\132\000\236\002\237\002\238\002\210\002\133\000\245\002\
\246\002\065\003\198\002\116\000\199\002\254\002\051\000\068\003\
\164\001\201\002\071\003\165\001\172\000\140\000\070\002\045\000\
\116\000\068\002\046\000\141\000\134\000\166\001\173\000\064\001\
\001\003\167\001\064\001\006\001\084\000\171\001\168\001\048\000\
\049\000\006\001\174\000\175\000\006\001\022\003\176\000\177\000\
\034\003\179\000\135\000\136\000\028\003\176\000\177\000\178\000\
\179\000\020\002\021\002\050\000\023\002\024\002\143\000\180\000\
\137\000\138\000\107\003\152\000\028\001\181\000\028\001\139\000\
\028\001\161\000\139\003\092\000\092\000\051\000\157\000\092\000\
\092\000\092\000\092\000\092\000\162\000\092\000\009\001\036\001\
\165\000\092\000\092\000\092\000\006\001\092\000\020\002\041\003\
\166\000\023\002\024\002\043\003\167\000\092\000\092\000\134\000\
\168\000\175\000\169\000\092\000\052\003\170\000\195\000\031\003\
\079\002\080\002\184\000\176\000\177\000\178\000\179\000\104\003\
\092\000\092\000\023\002\024\002\125\001\135\000\136\000\126\001\
\185\000\140\003\111\003\181\000\113\003\127\001\079\003\164\001\
\188\000\128\001\165\001\137\000\138\000\129\001\092\000\130\001\
\189\000\086\003\139\000\192\000\055\002\197\000\123\003\092\000\
\167\001\196\000\131\001\006\001\198\000\168\001\095\003\132\001\
\133\001\194\000\092\000\092\000\089\003\239\000\090\003\091\003\
\239\000\092\003\239\000\239\000\239\000\199\000\239\000\114\003\
\217\001\040\001\239\000\239\000\239\000\207\000\239\000\115\003\
\176\000\177\000\178\000\179\000\208\000\061\000\239\000\239\000\
\062\000\110\003\172\000\209\000\239\000\215\000\063\000\210\000\
\181\000\216\000\064\000\212\000\173\000\217\000\065\000\220\000\
\119\003\239\000\239\000\123\002\124\002\122\003\134\001\135\001\
\174\000\175\000\221\000\066\000\237\000\238\000\067\000\222\000\
\068\000\069\000\239\000\176\000\177\000\178\000\179\000\239\000\
\144\003\224\002\231\000\226\002\227\002\180\000\143\003\240\000\
\239\000\241\000\242\000\181\000\243\000\018\001\008\001\012\001\
\151\003\018\001\153\003\239\000\239\000\070\000\026\001\034\001\
\035\001\146\003\035\001\035\001\035\001\035\001\035\001\035\001\
\035\001\035\001\027\001\035\001\041\001\035\001\071\000\035\001\
\035\001\035\001\035\001\035\001\035\001\038\001\072\000\176\000\
\177\000\178\000\179\000\035\001\035\001\159\000\040\001\043\001\
\049\001\035\001\055\001\159\000\050\001\159\000\159\000\159\000\
\056\001\159\000\035\001\035\001\159\000\057\001\035\001\035\001\
\182\000\064\001\078\001\172\000\072\001\035\001\035\001\035\001\
\035\001\035\001\080\001\081\001\094\001\109\001\159\000\035\001\
\110\001\159\000\143\001\115\001\035\001\035\001\117\001\141\001\
\118\001\174\000\175\000\144\001\145\001\035\001\146\001\147\001\
\152\001\153\001\018\001\154\001\176\000\177\000\178\000\179\000\
\035\001\035\001\036\001\036\001\036\001\036\001\036\001\036\001\
\036\001\036\001\036\001\036\001\181\000\036\001\043\001\172\001\
\173\001\036\001\036\001\036\001\036\001\036\001\036\001\143\000\
\174\001\175\001\176\001\159\000\177\001\036\001\036\001\185\001\
\191\000\159\000\191\000\036\001\191\000\186\001\191\000\189\001\
\016\002\191\000\192\001\181\001\036\001\036\001\184\001\193\001\
\036\001\036\001\194\001\061\000\195\001\201\001\062\000\036\001\
\036\001\036\001\196\001\036\001\063\000\203\001\018\002\019\002\
\153\000\036\001\211\001\212\001\065\000\216\001\036\001\036\001\
\218\001\020\002\021\002\022\002\023\002\024\002\227\001\036\001\
\179\000\066\000\226\001\025\002\067\000\229\001\068\000\069\000\
\237\001\026\002\036\001\036\001\040\001\040\001\040\001\040\001\
\040\001\040\001\040\001\040\001\040\001\040\001\238\001\040\001\
\042\001\240\001\241\001\040\001\040\001\040\001\040\001\040\001\
\040\001\243\001\242\001\070\000\244\001\246\001\247\001\040\001\
\040\001\000\002\134\000\252\001\253\001\040\001\254\001\169\000\
\001\002\169\000\169\000\169\000\071\000\169\000\040\001\040\001\
\169\000\005\002\040\001\040\001\072\000\095\001\096\001\097\001\
\135\000\136\000\040\001\040\001\001\000\002\000\003\000\004\000\
\005\000\012\002\169\000\040\001\008\002\013\002\137\000\138\000\
\040\001\040\001\014\002\100\001\101\001\139\000\102\001\032\002\
\015\002\040\001\027\002\028\002\103\001\029\002\036\002\037\002\
\104\001\038\002\051\002\191\001\040\001\040\001\040\002\041\001\
\041\001\041\001\041\001\041\001\041\001\041\001\041\001\041\001\
\041\001\105\001\041\001\045\001\220\001\134\000\041\001\041\001\
\041\001\041\001\041\001\041\001\054\002\100\000\061\002\169\000\
\100\000\063\002\041\001\041\001\068\002\169\000\172\000\072\002\
\041\001\073\002\100\000\135\000\136\000\100\000\100\000\074\002\
\173\000\041\001\041\001\098\002\178\001\041\001\041\001\084\002\
\091\002\137\000\138\000\094\002\174\000\175\000\041\001\095\002\
\139\000\100\000\096\002\107\002\111\002\106\002\041\001\176\000\
\177\000\178\000\179\000\041\001\041\001\109\002\113\002\115\002\
\118\002\180\000\116\002\100\000\041\001\122\002\125\002\181\000\
\020\002\021\002\022\002\023\002\024\002\129\002\130\002\041\001\
\041\001\043\001\043\001\043\001\043\001\043\001\043\001\043\001\
\043\001\043\001\043\001\120\002\043\001\044\001\133\002\134\000\
\043\001\043\001\043\001\043\001\043\001\043\001\231\001\205\002\
\139\002\141\002\144\002\145\002\043\001\043\001\153\002\154\002\
\158\002\159\002\043\001\160\002\162\002\135\000\136\000\172\000\
\164\002\167\002\170\002\043\001\043\001\174\002\173\002\043\001\
\043\001\173\000\215\000\137\000\138\000\215\000\175\002\177\002\
\196\002\181\002\139\000\191\002\187\002\174\000\175\000\215\000\
\043\001\203\002\215\000\215\000\192\002\043\001\043\001\197\002\
\176\000\177\000\178\000\179\000\207\002\208\002\043\001\209\002\
\211\002\213\002\180\000\216\002\221\002\223\002\215\000\205\000\
\181\000\043\001\043\001\042\001\042\001\042\001\042\001\042\001\
\042\001\042\001\042\001\042\001\042\001\234\002\042\001\048\001\
\215\000\247\002\042\001\042\001\042\001\042\001\042\001\042\001\
\087\003\233\002\003\003\235\002\255\002\005\003\042\001\042\001\
\014\003\017\003\024\003\194\000\042\001\194\000\006\003\194\000\
\016\002\194\000\134\000\007\003\194\000\042\001\042\001\045\002\
\018\003\042\001\042\001\020\003\019\003\021\003\036\003\027\003\
\037\003\038\003\048\003\039\003\049\003\050\003\018\002\019\002\
\135\000\136\000\042\001\051\003\053\003\055\003\056\003\042\001\
\054\003\020\002\021\002\022\002\023\002\024\002\137\000\138\000\
\042\001\057\003\058\003\025\002\059\003\139\000\060\003\046\002\
\064\003\026\002\067\003\042\001\042\001\075\003\045\001\045\001\
\045\001\045\001\045\001\045\001\045\001\045\001\045\001\045\001\
\073\003\045\001\047\001\078\003\082\003\045\001\045\001\045\001\
\045\001\045\001\045\001\084\003\068\002\088\003\093\003\096\003\
\098\003\045\001\045\001\124\003\117\003\172\000\105\003\045\001\
\112\003\102\003\016\002\127\003\193\000\041\001\130\003\173\000\
\045\001\128\003\132\003\136\003\045\001\045\001\142\003\133\003\
\137\003\010\003\041\000\174\000\175\000\071\001\072\001\075\001\
\018\002\019\002\050\001\135\000\136\000\045\001\176\000\177\000\
\178\000\179\000\045\001\020\002\021\002\022\002\023\002\024\002\
\180\000\137\000\138\000\045\001\125\000\025\002\181\000\080\001\
\139\000\084\000\173\000\026\002\129\000\062\000\045\001\045\001\
\044\001\044\001\044\001\044\001\044\001\044\001\044\001\044\001\
\044\001\044\001\123\000\044\001\030\003\104\000\213\000\044\001\
\044\001\044\001\044\001\044\001\171\000\172\000\200\000\106\000\
\053\001\060\000\172\000\044\001\044\001\089\000\134\003\173\000\
\179\000\044\001\198\000\068\001\173\000\092\000\032\001\066\001\
\030\001\127\000\044\001\174\000\175\000\232\000\044\001\044\001\
\174\000\175\000\206\000\092\000\135\000\136\000\176\000\177\000\
\178\000\179\000\141\000\176\000\177\000\178\000\179\000\044\001\
\180\000\177\000\137\000\138\000\044\001\180\000\181\000\041\000\
\059\001\139\000\079\000\181\000\074\000\044\001\175\000\204\000\
\084\000\182\000\202\000\014\001\202\002\015\001\198\001\124\001\
\044\001\044\001\048\001\048\001\048\001\048\001\048\001\048\001\
\048\001\048\001\048\001\048\001\101\002\048\001\031\002\149\001\
\236\001\048\001\048\001\048\001\048\001\048\001\241\002\172\000\
\142\001\101\003\085\002\097\003\019\001\048\001\048\001\004\003\
\055\001\173\000\206\001\048\001\119\002\154\003\094\003\008\003\
\016\002\088\002\232\002\013\001\000\000\174\000\175\000\013\003\
\048\001\048\001\000\000\000\000\000\000\000\000\000\000\000\000\
\176\000\177\000\178\000\179\000\000\000\000\000\018\002\019\002\
\000\000\048\001\180\000\000\000\000\000\000\000\048\001\000\000\
\181\000\020\002\021\002\022\002\023\002\024\002\000\000\048\001\
\049\001\000\000\000\000\025\002\000\000\000\000\000\000\000\000\
\000\000\026\002\048\001\048\001\000\000\047\001\047\001\047\001\
\047\001\047\001\047\001\047\001\047\001\047\001\047\001\000\000\
\047\001\000\000\000\000\000\000\047\001\047\001\047\001\047\001\
\047\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\047\001\047\001\000\000\000\000\000\000\000\000\047\001\000\000\
\056\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\047\001\047\001\050\001\050\001\050\001\
\050\001\050\001\050\001\050\001\050\001\050\001\050\001\000\000\
\050\001\000\000\000\000\000\000\050\001\050\001\050\001\050\001\
\050\001\047\001\000\000\000\000\000\000\000\000\000\000\000\000\
\050\001\050\001\047\001\000\000\000\000\000\000\050\001\000\000\
\060\001\000\000\000\000\000\000\000\000\047\001\047\001\000\000\
\000\000\000\000\000\000\050\001\050\001\000\000\000\000\000\000\
\000\000\000\000\000\000\053\001\053\001\053\001\053\001\053\001\
\053\001\053\001\053\001\053\001\053\001\000\000\053\001\000\000\
\000\000\050\001\053\001\053\001\053\001\053\001\053\001\000\000\
\000\000\000\000\050\001\000\000\000\000\000\000\053\001\053\001\
\062\001\000\000\000\000\000\000\053\001\050\001\050\001\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\053\001\053\001\059\001\059\001\059\001\059\001\059\001\
\059\001\059\001\059\001\059\001\059\001\000\000\059\001\000\000\
\000\000\000\000\059\001\059\001\059\001\059\001\059\001\053\001\
\000\000\000\000\000\000\000\000\000\000\000\000\059\001\059\001\
\053\001\000\000\000\000\000\000\059\001\000\000\238\000\000\000\
\000\000\000\000\000\000\053\001\053\001\000\000\000\000\000\000\
\000\000\059\001\059\001\055\001\055\001\055\001\055\001\055\001\
\055\001\055\001\055\001\055\001\055\001\000\000\055\001\000\000\
\000\000\000\000\055\001\055\001\055\001\055\001\055\001\059\001\
\000\000\000\000\000\000\000\000\000\000\000\000\055\001\055\001\
\059\001\000\000\000\000\000\000\055\001\084\000\000\000\000\000\
\000\000\000\000\000\000\059\001\059\001\000\000\000\000\000\000\
\000\000\055\001\055\001\049\001\049\001\049\001\049\001\049\001\
\049\001\049\001\049\001\049\001\049\001\000\000\049\001\000\000\
\000\000\000\000\049\001\049\001\049\001\049\001\049\001\055\001\
\000\000\000\000\000\000\000\000\000\000\000\000\049\001\049\001\
\055\001\000\000\000\000\000\000\049\001\061\001\000\000\000\000\
\000\000\000\000\000\000\055\001\055\001\000\000\000\000\000\000\
\000\000\049\001\049\001\056\001\056\001\056\001\056\001\056\001\
\056\001\056\001\056\001\056\001\056\001\000\000\056\001\000\000\
\000\000\000\000\056\001\056\001\056\001\056\001\056\001\049\001\
\000\000\000\000\000\000\000\000\000\000\000\000\056\001\056\001\
\049\001\000\000\000\000\092\000\056\001\000\000\000\000\000\000\
\000\000\000\000\000\000\049\001\049\001\000\000\000\000\000\000\
\000\000\056\001\056\001\060\001\060\001\060\001\060\001\060\001\
\060\001\060\001\060\001\060\001\060\001\000\000\060\001\000\000\
\000\000\000\000\060\001\060\001\060\001\060\001\000\000\056\001\
\000\000\000\000\000\000\000\000\000\000\000\000\060\001\060\001\
\056\001\000\000\013\001\000\000\060\001\000\000\000\000\000\000\
\000\000\000\000\000\000\056\001\056\001\000\000\000\000\000\000\
\000\000\060\001\060\001\000\000\062\001\062\001\062\001\000\000\
\062\001\000\000\062\001\062\001\062\001\000\000\062\001\000\000\
\000\000\111\000\062\001\062\001\062\001\000\000\062\001\060\001\
\111\000\000\000\000\000\000\000\000\000\000\000\062\001\062\001\
\060\001\239\000\000\000\000\000\062\001\000\000\000\000\111\000\
\111\000\091\000\000\000\060\001\060\001\000\000\000\000\000\000\
\000\000\062\001\062\001\000\000\000\000\111\000\111\000\000\000\
\000\000\000\000\238\000\238\000\111\000\000\000\238\000\238\000\
\238\000\238\000\238\000\000\000\238\000\000\000\000\000\062\001\
\238\000\238\000\238\000\000\000\238\000\000\000\000\000\000\000\
\062\001\084\000\000\000\000\000\238\000\238\000\000\000\018\002\
\019\002\000\000\238\000\062\001\062\001\000\000\000\000\000\000\
\000\000\000\000\020\002\021\002\022\002\023\002\024\002\238\000\
\238\000\084\000\084\000\016\002\025\002\084\000\084\000\084\000\
\084\000\084\000\026\002\084\000\000\000\000\000\000\000\084\000\
\084\000\084\000\000\000\084\000\000\000\238\000\000\000\000\000\
\241\000\018\002\019\002\084\000\084\000\000\000\238\000\000\000\
\000\000\084\000\000\000\000\000\020\002\021\002\022\002\023\002\
\024\002\238\000\238\000\000\000\000\000\000\000\084\000\084\000\
\000\000\061\001\061\001\061\001\026\002\061\001\000\000\061\001\
\061\001\061\001\000\000\061\001\000\000\152\003\000\000\061\001\
\061\001\061\001\000\000\061\001\084\000\000\000\000\000\006\001\
\000\000\000\000\000\000\061\001\061\001\084\000\000\000\000\000\
\000\000\061\001\000\000\135\000\136\000\000\000\000\000\000\000\
\084\000\084\000\000\000\000\000\000\000\000\000\061\001\061\001\
\092\000\137\000\138\000\092\000\092\000\092\000\092\000\092\000\
\139\000\092\000\000\000\000\000\000\000\092\000\092\000\092\000\
\000\000\092\000\000\000\000\000\061\001\000\000\255\000\000\000\
\000\000\092\000\092\000\000\000\000\000\061\001\000\000\092\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\061\001\061\001\000\000\000\000\092\000\092\000\013\001\013\001\
\000\000\000\000\013\001\000\000\013\001\013\001\013\001\000\000\
\013\001\000\000\000\000\000\000\013\001\013\001\013\001\000\000\
\013\001\000\000\092\000\000\000\000\000\012\001\000\000\000\000\
\013\001\013\001\000\000\092\000\000\000\000\000\013\001\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\092\000\092\000\
\000\000\000\000\000\000\013\001\013\001\239\000\239\000\000\000\
\000\000\239\000\000\000\239\000\239\000\239\000\000\000\239\000\
\000\000\000\000\000\000\239\000\239\000\239\000\000\000\239\000\
\000\000\013\001\000\000\000\000\002\001\000\000\000\000\239\000\
\239\000\000\000\013\001\000\000\000\000\239\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\013\001\013\001\000\000\
\000\000\000\000\239\000\239\000\000\000\084\000\084\000\000\000\
\000\000\084\000\000\000\084\000\084\000\084\000\000\000\084\000\
\000\000\000\000\000\000\084\000\084\000\084\000\000\000\084\000\
\239\000\000\000\000\000\016\001\000\000\000\000\000\000\084\000\
\084\000\239\000\000\000\000\000\000\000\084\000\000\000\000\000\
\000\000\000\000\000\000\000\000\239\000\239\000\000\000\000\000\
\000\000\000\000\084\000\084\000\241\000\241\000\000\000\000\000\
\000\000\000\000\241\000\241\000\241\000\000\000\241\000\000\000\
\000\000\000\000\241\000\241\000\241\000\000\000\241\000\000\000\
\084\000\000\000\015\001\000\000\000\000\000\000\241\000\241\000\
\000\000\084\000\000\000\000\000\241\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\084\000\084\000\000\000\000\000\
\000\000\241\000\241\000\006\001\006\001\000\000\000\000\000\000\
\000\000\006\001\006\001\006\001\000\000\006\001\000\000\000\000\
\000\000\006\001\006\001\006\001\000\000\006\001\000\000\241\000\
\000\000\017\001\000\000\000\000\000\000\006\001\006\001\000\000\
\241\000\000\000\000\000\006\001\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\241\000\241\000\000\000\000\000\000\000\
\006\001\006\001\255\000\255\000\000\000\000\000\255\000\000\000\
\255\000\255\000\000\000\000\000\255\000\000\000\000\000\000\000\
\255\000\255\000\255\000\000\000\255\000\000\000\006\001\000\000\
\014\001\000\000\000\000\000\000\255\000\255\000\000\000\006\001\
\000\000\000\000\255\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\006\001\006\001\000\000\000\000\000\000\255\000\
\255\000\012\001\012\001\000\000\000\000\000\000\000\000\012\001\
\012\001\012\001\000\000\012\001\000\000\000\000\000\000\012\001\
\012\001\012\001\000\000\012\001\000\000\255\000\000\000\017\001\
\000\000\000\000\000\000\012\001\012\001\000\000\255\000\000\000\
\000\000\012\001\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\255\000\255\000\000\000\000\000\000\000\012\001\012\001\
\002\001\002\001\000\000\000\000\002\001\000\000\002\001\002\001\
\000\000\000\000\002\001\000\000\000\000\000\000\002\001\002\001\
\002\001\000\000\002\001\000\000\012\001\000\000\000\000\000\000\
\000\000\000\000\002\001\002\001\111\000\012\001\000\000\000\000\
\002\001\112\000\000\000\000\000\000\000\000\000\000\000\000\000\
\012\001\012\001\000\000\000\000\000\000\002\001\002\001\016\001\
\016\001\000\000\000\000\000\000\000\000\016\001\016\001\016\001\
\000\000\016\001\000\000\000\000\000\000\016\001\016\001\016\001\
\000\000\016\001\000\000\002\001\000\000\000\000\000\000\000\000\
\000\000\016\001\016\001\000\000\002\001\000\000\000\000\016\001\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\002\001\
\002\001\000\000\000\000\000\000\016\001\016\001\015\001\015\001\
\000\000\000\000\015\001\000\000\015\001\015\001\000\000\000\000\
\015\001\000\000\000\000\000\000\015\001\015\001\015\001\000\000\
\015\001\000\000\016\001\000\000\000\000\000\000\000\000\000\000\
\015\001\015\001\000\000\016\001\000\000\000\000\015\001\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\016\001\016\001\
\000\000\000\000\000\000\015\001\015\001\017\001\017\001\000\000\
\000\000\017\001\000\000\017\001\017\001\017\001\000\000\017\001\
\000\000\000\000\000\000\017\001\017\001\017\001\000\000\000\000\
\000\000\015\001\000\000\000\000\000\000\000\000\000\000\017\001\
\017\001\000\000\015\001\000\000\000\000\017\001\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\015\001\015\001\000\000\
\000\000\000\000\017\001\017\001\014\001\014\001\000\000\000\000\
\000\000\000\000\014\001\014\001\014\001\000\000\014\001\000\000\
\000\000\000\000\014\001\014\001\014\001\000\000\014\001\000\000\
\017\001\000\000\000\000\000\000\000\000\000\000\014\001\014\001\
\000\000\017\001\000\000\000\000\014\001\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\017\001\017\001\000\000\000\000\
\000\000\014\001\014\001\017\001\017\001\000\000\000\000\000\000\
\000\000\017\001\017\001\017\001\000\000\017\001\000\000\000\000\
\000\000\017\001\017\001\017\001\000\000\000\000\000\000\014\001\
\000\000\000\000\000\000\000\000\000\000\017\001\017\001\000\000\
\014\001\000\000\000\000\017\001\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\014\001\014\001\000\000\000\000\000\000\
\017\001\017\001\000\000\000\000\000\000\000\000\000\000\111\000\
\000\000\111\000\111\000\111\000\112\000\111\000\112\000\112\000\
\112\000\000\000\112\000\000\000\000\000\000\000\017\001\000\000\
\000\000\000\000\000\000\111\000\000\000\000\000\000\000\017\001\
\112\000\000\000\111\000\000\000\000\000\000\000\172\000\112\000\
\000\000\000\000\017\001\017\001\000\000\000\000\111\000\111\000\
\111\000\111\000\111\000\112\000\112\000\112\000\112\000\112\000\
\000\000\000\000\000\000\000\000\174\000\175\000\000\000\111\000\
\111\000\000\000\000\000\000\000\000\000\112\000\111\000\176\000\
\177\000\178\000\179\000\112\000\000\000\000\000\000\000\000\000\
\000\000\180\000\000\000\111\000\000\000\244\000\000\000\181\000\
\112\000\178\001\000\000\245\000\246\000\247\000\000\000\248\000\
\000\000\000\000\000\000\249\000\250\000\251\000\000\000\000\000\
\000\000\000\000\000\000\000\000\244\000\000\000\000\000\000\000\
\252\000\048\001\245\000\246\000\247\000\253\000\248\000\000\000\
\000\000\000\000\249\000\250\000\251\000\000\000\000\000\000\000\
\000\000\000\000\254\000\255\000\000\000\172\000\000\000\252\000\
\000\000\000\000\000\000\000\000\253\000\000\000\000\000\173\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\001\254\000\255\000\174\000\175\000\000\000\000\000\000\000\
\000\000\001\001\000\000\000\000\000\000\000\000\176\000\177\000\
\178\000\179\000\000\000\244\000\002\001\003\001\000\000\000\001\
\180\000\245\000\246\000\247\000\000\000\161\001\181\000\000\000\
\001\001\249\000\250\000\251\000\146\000\000\000\146\000\146\000\
\146\000\000\000\146\000\002\001\003\001\146\000\252\000\000\000\
\000\000\000\000\000\000\253\000\000\000\000\000\000\000\000\000\
\146\000\000\000\000\000\000\000\000\000\000\000\000\000\146\000\
\254\000\255\000\146\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\146\000\146\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\001\000\000\
\000\000\146\000\146\000\146\000\146\000\000\000\143\000\001\001\
\143\000\143\000\143\000\146\000\143\000\000\000\000\000\143\000\
\000\000\146\000\002\001\003\001\000\000\000\000\172\000\000\000\
\000\000\000\000\143\000\000\000\146\000\228\001\000\000\000\000\
\173\000\143\000\146\000\000\000\143\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\174\000\175\000\000\000\000\000\
\143\000\143\000\000\000\000\000\000\000\000\000\000\000\176\000\
\177\000\178\000\179\000\143\000\143\000\143\000\143\000\000\000\
\147\000\180\000\147\000\147\000\147\000\143\000\147\000\181\000\
\000\000\147\000\000\000\143\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\147\000\000\000\143\000\000\000\
\000\000\000\000\000\000\147\000\143\000\168\000\147\000\168\000\
\000\000\168\000\000\000\168\000\000\000\000\000\168\000\000\000\
\000\000\000\000\147\000\147\000\000\000\000\000\000\000\000\000\
\000\000\168\000\000\000\000\000\000\000\147\000\147\000\147\000\
\168\000\000\000\000\000\168\000\000\000\000\000\000\000\147\000\
\000\000\000\000\000\000\000\000\000\000\147\000\000\000\168\000\
\168\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\147\000\000\000\168\000\168\000\168\000\168\000\147\000\152\000\
\000\000\152\000\152\000\152\000\168\000\152\000\000\000\000\000\
\152\000\000\000\168\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\152\000\000\000\168\000\000\000\000\000\
\000\000\000\000\152\000\168\000\153\000\152\000\153\000\153\000\
\153\000\000\000\153\000\000\000\000\000\153\000\000\000\000\000\
\000\000\152\000\152\000\000\000\000\000\000\000\000\000\000\000\
\153\000\000\000\000\000\000\000\000\000\152\000\152\000\153\000\
\000\000\151\000\153\000\151\000\151\000\151\000\152\000\151\000\
\000\000\000\000\151\000\000\000\152\000\000\000\153\000\153\000\
\000\000\000\000\000\000\000\000\000\000\151\000\000\000\152\000\
\000\000\000\000\000\000\153\000\151\000\152\000\150\000\151\000\
\150\000\150\000\150\000\153\000\150\000\000\000\000\000\150\000\
\000\000\153\000\000\000\151\000\151\000\000\000\000\000\000\000\
\000\000\000\000\150\000\000\000\153\000\000\000\000\000\000\000\
\000\000\150\000\153\000\157\000\150\000\157\000\157\000\157\000\
\151\000\157\000\000\000\000\000\157\000\000\000\151\000\000\000\
\150\000\150\000\000\000\000\000\000\000\000\000\000\000\157\000\
\000\000\151\000\000\000\000\000\000\000\000\000\157\000\151\000\
\156\000\157\000\156\000\156\000\156\000\150\000\156\000\000\000\
\146\000\156\000\000\000\146\000\000\000\157\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\150\000\000\000\
\000\000\146\000\000\000\156\000\150\000\000\000\156\000\000\000\
\146\000\000\000\157\000\146\000\000\000\000\000\000\000\000\000\
\000\000\000\000\156\000\000\000\000\000\205\001\000\000\146\000\
\146\000\000\000\132\002\157\000\000\000\000\000\000\000\000\000\
\000\000\157\000\146\000\146\000\146\000\146\000\000\000\156\000\
\000\000\172\000\000\000\000\000\146\000\168\002\016\002\000\000\
\000\000\000\000\146\000\173\000\000\000\000\000\000\000\000\000\
\156\000\000\000\000\000\000\000\000\000\146\000\156\000\174\000\
\175\000\016\002\000\000\146\000\018\002\019\002\000\000\000\000\
\000\000\169\002\176\000\177\000\178\000\179\000\000\000\020\002\
\021\002\022\002\023\002\024\002\180\000\000\000\000\000\018\002\
\019\002\025\002\181\000\231\002\000\000\016\002\000\000\026\002\
\000\000\000\000\020\002\021\002\022\002\023\002\024\002\000\000\
\000\000\000\000\000\000\000\000\025\002\000\000\000\000\016\002\
\000\000\000\000\026\002\018\002\019\002\062\003\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\020\002\021\002\
\022\002\023\002\024\002\000\000\000\000\018\002\019\002\016\002\
\025\002\000\000\000\000\000\000\000\000\000\000\026\002\000\000\
\020\002\021\002\022\002\023\002\024\002\000\000\000\000\000\000\
\000\000\000\000\025\002\000\000\000\000\018\002\019\002\000\000\
\026\002\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\020\002\021\002\022\002\023\002\024\002\000\000\007\000\008\000\
\009\000\010\000\025\002\000\000\000\000\011\000\012\000\000\000\
\026\002\013\000\014\000\015\000\016\000\017\000\000\000\018\000\
\000\000\000\000\244\002\000\000\000\000\000\000\019\000\000\000\
\000\000\000\000\020\000\021\000\000\000\000\000\000\000\000\000\
\016\002\000\000\000\000\022\000\023\000\000\000\024\000\000\000\
\000\000\025\000\026\000\027\000\028\000\000\000\000\000\029\000\
\000\000\081\003\030\000\031\000\000\000\000\000\018\002\019\002\
\000\000\000\000\000\000\000\000\032\000\033\000\034\000\172\000\
\035\000\020\002\021\002\022\002\023\002\024\002\000\000\016\003\
\000\000\173\000\000\000\025\002\046\003\000\000\000\000\000\000\
\000\000\026\002\016\002\000\000\000\000\174\000\175\000\016\002\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\176\000\177\000\178\000\179\000\047\003\000\000\000\000\000\000\
\018\002\019\002\180\000\000\000\000\000\018\002\019\002\016\002\
\181\000\000\000\000\000\020\002\021\002\022\002\023\002\024\002\
\020\002\021\002\022\002\023\002\024\002\025\002\053\002\000\000\
\016\002\000\000\025\002\026\002\000\000\018\002\019\002\000\000\
\026\002\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\020\002\021\002\022\002\023\002\024\002\000\000\018\002\019\002\
\000\000\000\000\025\002\138\002\000\000\016\002\000\000\000\000\
\026\002\020\002\021\002\022\002\023\002\024\002\000\000\000\000\
\000\000\000\000\000\000\025\002\000\000\044\003\016\002\000\000\
\000\000\026\002\000\000\018\002\019\002\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\020\002\021\002\
\022\002\023\002\024\002\000\000\018\002\019\002\016\002\000\000\
\025\002\000\000\000\000\000\000\000\000\017\002\026\002\020\002\
\021\002\022\002\023\002\024\002\000\000\000\000\000\000\016\002\
\000\000\025\002\000\000\000\000\018\002\019\002\040\003\026\002\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\020\002\
\021\002\022\002\023\002\024\002\000\000\018\002\019\002\016\002\
\000\000\025\002\000\000\000\000\172\000\000\000\000\000\026\002\
\020\002\021\002\022\002\023\002\024\002\000\000\173\000\000\000\
\000\000\000\000\025\002\000\000\000\000\018\002\019\002\000\000\
\026\002\000\000\174\000\175\000\000\000\000\000\000\000\000\000\
\020\002\021\002\022\002\023\002\024\002\176\000\177\000\178\000\
\179\000\000\000\025\002\000\000\000\000\000\000\000\000\180\000\
\026\002\000\000\000\000\000\000\000\000\181\000"

let yycheck = "\005\000\
\013\000\009\000\160\000\002\000\168\000\252\000\174\001\210\000\
\000\000\057\000\016\000\059\000\172\001\000\000\119\000\000\000\
\110\001\068\003\013\000\025\003\015\000\008\001\004\001\000\000\
\019\000\000\000\074\000\208\000\209\000\059\002\155\001\009\001\
\216\001\016\000\218\001\014\001\220\001\148\000\196\000\118\000\
\119\000\199\000\017\001\038\000\074\002\058\003\151\000\003\001\
\090\003\091\003\056\000\099\000\058\000\101\000\102\000\004\001\
\107\003\006\001\216\000\054\000\003\001\002\001\003\001\004\001\
\005\001\047\001\003\001\015\001\012\000\004\001\170\000\015\000\
\151\000\014\001\122\000\016\001\014\001\018\001\014\001\190\000\
\131\003\019\001\023\001\001\001\007\001\185\000\004\001\093\000\
\004\001\030\001\006\001\047\001\010\001\092\000\038\000\074\001\
\014\001\075\001\047\001\017\000\018\001\005\002\084\001\021\000\
\146\003\080\001\024\000\025\000\091\000\220\000\221\000\124\003\
\047\001\031\001\003\001\145\002\059\001\024\001\036\001\037\001\
\126\000\062\001\059\001\072\001\000\000\131\000\132\000\133\000\
\134\000\135\000\136\000\047\001\243\000\139\000\023\001\084\001\
\059\001\006\002\050\002\244\000\131\001\147\003\014\001\237\001\
\014\001\250\000\005\001\065\001\030\003\006\001\091\001\040\001\
\014\001\028\001\061\001\067\002\005\001\064\001\007\001\063\001\
\028\001\031\001\009\001\014\001\076\002\244\000\036\001\063\001\
\084\001\026\001\059\001\250\000\090\001\252\000\014\001\188\000\
\003\001\102\001\006\001\007\001\031\001\009\001\005\001\090\001\
\007\001\036\001\004\001\027\001\050\001\184\000\189\000\197\000\
\191\000\040\001\193\000\067\001\195\000\066\002\185\001\081\003\
\059\001\003\001\030\001\078\001\073\002\000\000\003\001\003\001\
\152\000\005\001\078\001\007\001\030\001\212\000\036\001\198\000\
\023\001\161\000\217\000\047\001\090\001\014\001\166\000\222\000\
\051\001\169\000\170\000\023\001\172\000\173\000\174\000\175\000\
\176\000\177\000\178\000\219\000\180\000\181\000\142\002\090\001\
\158\000\185\000\241\000\065\001\009\001\047\001\006\001\075\001\
\192\000\051\001\006\001\014\001\059\001\014\001\051\001\157\001\
\158\001\252\000\019\001\003\001\004\001\003\001\004\001\059\001\
\182\000\030\001\013\001\251\000\186\000\061\001\030\001\063\001\
\141\002\009\001\030\001\144\002\194\000\008\001\013\001\008\001\
\118\002\209\002\023\001\211\002\003\001\045\001\005\001\006\001\
\007\001\045\001\009\001\041\001\003\001\074\001\005\001\038\001\
\007\001\040\001\014\001\009\001\043\001\093\001\049\001\047\001\
\006\001\047\001\066\001\028\001\131\001\211\001\066\001\030\001\
\023\001\004\001\049\001\023\001\241\001\006\001\059\001\054\002\
\008\001\064\001\114\001\042\001\043\001\044\001\061\002\029\001\
\030\001\178\001\122\001\003\001\014\001\005\001\006\001\007\001\
\046\001\009\001\029\001\006\001\012\001\030\001\165\001\008\001\
\000\003\095\001\096\001\097\001\059\001\059\001\060\001\059\001\
\004\001\000\000\075\001\178\001\045\001\078\001\030\001\014\001\
\079\001\007\002\018\003\030\001\020\003\030\001\005\001\055\001\
\046\001\017\003\014\001\023\001\115\001\003\001\117\001\118\001\
\003\001\066\001\216\002\014\001\006\001\059\001\060\001\003\001\
\023\001\005\001\006\001\007\001\006\001\009\001\014\001\183\001\
\131\001\023\001\056\001\057\001\023\001\159\000\141\001\003\001\
\004\001\079\001\130\001\165\000\030\001\167\000\045\001\046\001\
\014\001\201\001\030\001\083\001\030\001\069\003\060\002\045\001\
\046\001\089\001\045\001\046\001\059\001\060\001\042\001\043\001\
\044\001\153\001\154\001\066\001\008\001\059\001\060\001\122\002\
\059\001\060\001\169\002\009\001\066\001\187\001\084\002\066\001\
\168\001\006\001\014\001\186\001\009\001\006\001\066\001\103\003\
\104\003\006\001\030\001\009\001\009\001\181\001\036\003\009\001\
\030\001\039\003\014\001\079\001\053\002\204\001\004\001\009\001\
\003\001\030\001\005\001\006\001\007\001\030\001\009\001\117\003\
\030\001\030\001\009\001\068\002\030\001\023\001\016\001\000\000\
\018\001\023\001\008\001\226\001\023\001\228\001\053\002\054\002\
\231\001\005\001\136\003\030\001\059\002\009\001\061\002\030\001\
\022\001\185\001\091\002\045\001\046\001\068\002\084\003\042\001\
\043\001\044\001\045\001\074\002\042\001\252\001\253\001\254\001\
\006\001\059\001\060\001\030\001\007\001\205\001\012\001\009\001\
\066\001\167\002\001\001\014\001\091\002\004\001\004\001\066\001\
\006\001\001\002\017\002\010\001\047\001\012\003\057\001\014\001\
\008\002\003\001\004\001\018\001\079\001\013\002\059\001\060\001\
\016\002\197\002\018\002\019\002\020\002\021\002\022\002\046\001\
\031\001\025\002\026\002\027\002\009\001\036\001\037\001\009\001\
\014\001\223\001\009\001\029\001\059\001\060\001\014\001\023\001\
\000\000\003\001\003\001\005\001\005\001\006\001\007\001\009\001\
\009\001\030\001\145\002\012\001\030\001\047\001\056\001\030\001\
\177\002\059\001\065\001\233\002\014\001\045\001\046\001\063\002\
\064\002\089\002\014\001\046\001\016\001\030\001\018\001\004\001\
\033\001\006\001\169\002\059\001\060\001\056\001\057\001\058\001\
\059\001\060\001\177\002\090\001\003\001\180\002\005\001\006\001\
\007\001\005\001\009\001\106\002\008\001\072\001\109\002\006\001\
\001\001\003\001\113\002\004\001\003\001\007\001\005\001\004\001\
\007\001\066\001\040\002\056\001\057\001\014\001\125\002\030\001\
\017\001\018\001\129\002\056\001\057\001\058\001\059\001\060\001\
\023\001\185\002\083\001\042\001\043\001\044\001\045\001\007\001\
\089\001\009\001\130\002\072\001\037\001\000\000\094\002\004\001\
\200\002\059\001\138\002\139\002\140\002\101\002\004\001\158\002\
\159\002\010\003\080\002\066\001\007\001\164\002\055\001\016\003\
\001\001\087\002\019\003\004\001\023\001\004\001\006\001\001\001\
\079\001\009\001\004\001\014\001\023\001\014\001\033\001\006\001\
\168\002\018\001\009\001\010\003\014\001\012\003\023\001\017\001\
\018\001\016\003\045\001\046\001\019\003\196\002\056\001\057\001\
\213\002\059\001\045\001\046\001\203\002\056\001\057\001\058\001\
\059\001\056\001\057\001\037\001\059\001\060\001\006\001\066\001\
\059\001\060\001\067\003\006\001\003\001\072\001\005\001\066\001\
\007\001\059\001\128\003\004\001\005\001\055\001\014\001\008\001\
\009\001\010\001\011\001\012\001\014\001\014\001\089\001\000\000\
\004\001\018\001\019\001\020\001\067\003\022\001\056\001\231\002\
\004\001\059\001\060\001\235\002\004\001\030\001\031\001\023\001\
\014\001\046\001\004\001\036\001\244\002\009\001\030\001\207\002\
\082\001\083\001\003\001\056\001\057\001\058\001\059\001\063\003\
\049\001\050\001\059\001\060\001\001\001\045\001\046\001\004\001\
\009\001\130\003\074\003\072\001\076\003\010\001\029\003\001\001\
\009\001\014\001\004\001\059\001\060\001\018\001\071\001\020\001\
\030\001\040\003\066\001\009\001\014\001\009\001\094\003\080\001\
\018\001\047\001\031\001\130\003\003\001\023\001\053\003\036\001\
\037\001\003\001\091\001\092\001\044\003\005\001\046\003\047\003\
\008\001\049\003\010\001\011\001\012\001\047\001\014\001\077\003\
\005\001\000\000\018\001\019\001\020\001\014\001\022\001\078\003\
\056\001\057\001\058\001\059\001\004\001\001\001\030\001\031\001\
\004\001\073\003\023\001\004\001\036\001\023\001\010\001\009\001\
\072\001\047\001\014\001\030\001\033\001\030\001\018\001\004\001\
\088\003\049\001\050\001\246\001\247\001\093\003\091\001\092\001\
\045\001\046\001\004\001\031\001\014\001\014\001\034\001\030\001\
\036\001\037\001\014\001\056\001\057\001\058\001\059\001\071\001\
\134\003\121\002\074\001\123\002\124\002\066\001\133\003\014\001\
\080\001\030\001\009\001\072\001\004\001\085\001\009\001\015\001\
\150\003\005\001\152\003\091\001\092\001\065\001\018\001\009\001\
\003\001\137\003\005\001\006\001\007\001\008\001\009\001\010\001\
\011\001\012\001\018\001\014\001\000\000\014\001\082\001\018\001\
\019\001\020\001\021\001\022\001\023\001\030\001\090\001\056\001\
\057\001\058\001\059\001\030\001\031\001\004\001\030\001\030\001\
\005\001\036\001\023\001\003\001\006\001\005\001\006\001\007\001\
\047\001\009\001\045\001\046\001\012\001\047\001\049\001\050\001\
\083\001\030\001\014\001\023\001\018\001\056\001\057\001\058\001\
\059\001\060\001\004\001\014\001\009\001\018\001\030\001\066\001\
\023\001\033\001\009\001\030\001\071\001\072\001\030\001\075\001\
\030\001\045\001\046\001\023\001\023\001\080\001\023\001\023\001\
\014\001\004\001\085\001\004\001\056\001\057\001\058\001\059\001\
\091\001\092\001\003\001\004\001\005\001\006\001\007\001\008\001\
\009\001\010\001\011\001\012\001\072\001\014\001\000\000\014\001\
\018\001\018\001\019\001\020\001\021\001\022\001\023\001\006\001\
\018\001\014\001\014\001\083\001\014\001\030\001\031\001\009\001\
\003\001\089\001\005\001\036\001\007\001\030\001\009\001\005\001\
\023\001\012\001\005\001\085\001\045\001\046\001\074\001\003\001\
\049\001\050\001\023\001\001\001\007\001\005\001\004\001\056\001\
\057\001\058\001\003\001\060\001\010\001\005\001\045\001\046\001\
\014\001\066\001\047\001\005\001\018\001\005\001\071\001\072\001\
\005\001\056\001\057\001\058\001\059\001\060\001\005\001\080\001\
\059\001\031\001\030\001\066\001\034\001\054\001\036\001\037\001\
\006\001\072\001\091\001\092\001\003\001\004\001\005\001\006\001\
\007\001\008\001\009\001\010\001\011\001\012\001\014\001\014\001\
\000\000\014\001\004\001\018\001\019\001\020\001\021\001\022\001\
\023\001\003\001\014\001\065\001\005\001\005\001\005\001\030\001\
\031\001\005\001\023\001\030\001\030\001\036\001\030\001\003\001\
\006\001\005\001\006\001\007\001\082\001\009\001\045\001\046\001\
\012\001\014\001\049\001\050\001\090\001\042\001\043\001\044\001\
\045\001\046\001\057\001\058\001\001\000\002\000\003\000\004\000\
\005\000\014\001\030\001\066\001\059\001\004\001\059\001\060\001\
\071\001\072\001\014\001\001\001\002\001\066\001\004\001\016\001\
\014\001\080\001\085\001\005\001\010\001\005\001\016\001\016\001\
\014\001\005\001\074\001\005\001\091\001\092\001\006\001\003\001\
\004\001\005\001\006\001\007\001\008\001\009\001\010\001\011\001\
\012\001\031\001\014\001\000\000\005\001\023\001\018\001\019\001\
\020\001\021\001\022\001\023\001\006\001\001\001\059\001\083\001\
\004\001\040\001\030\001\031\001\009\001\089\001\023\001\072\001\
\036\001\004\001\014\001\045\001\046\001\017\001\018\001\004\001\
\033\001\045\001\046\001\013\001\008\001\049\001\050\001\047\001\
\023\001\059\001\060\001\023\001\045\001\046\001\058\001\018\001\
\066\001\037\001\007\001\005\001\055\001\030\001\066\001\056\001\
\057\001\058\001\059\001\071\001\072\001\030\001\030\001\005\001\
\023\001\066\001\006\001\055\001\080\001\012\001\030\001\072\001\
\056\001\057\001\058\001\059\001\060\001\030\001\023\001\091\001\
\092\001\003\001\004\001\005\001\006\001\007\001\008\001\009\001\
\010\001\011\001\012\001\049\001\014\001\000\000\005\001\023\001\
\018\001\019\001\020\001\021\001\022\001\023\001\030\001\007\001\
\040\001\004\001\004\001\004\001\030\001\031\001\018\001\018\001\
\074\001\030\001\036\001\006\001\006\001\045\001\046\001\023\001\
\030\001\047\001\005\001\045\001\046\001\007\001\047\001\049\001\
\050\001\033\001\001\001\059\001\060\001\004\001\003\001\014\001\
\030\001\005\001\066\001\014\001\018\001\045\001\046\001\014\001\
\066\001\030\001\017\001\018\001\014\001\071\001\072\001\014\001\
\056\001\057\001\058\001\059\001\023\001\014\001\080\001\018\001\
\018\001\003\001\066\001\003\001\005\001\018\001\037\001\029\001\
\072\001\091\001\092\001\003\001\004\001\005\001\006\001\007\001\
\008\001\009\001\010\001\011\001\012\001\005\001\014\001\000\000\
\055\001\014\001\018\001\019\001\020\001\021\001\022\001\023\001\
\007\001\047\001\006\001\009\001\007\001\014\001\030\001\031\001\
\005\001\005\001\075\001\003\001\036\001\005\001\014\001\007\001\
\023\001\009\001\023\001\014\001\012\001\045\001\046\001\014\001\
\007\001\049\001\050\001\005\001\081\001\005\001\023\001\075\001\
\018\001\007\001\005\001\012\001\009\001\005\001\045\001\046\001\
\045\001\046\001\066\001\005\001\075\001\009\001\007\001\071\001\
\023\001\056\001\057\001\058\001\059\001\060\001\059\001\060\001\
\080\001\023\001\023\001\066\001\009\001\066\001\007\001\050\001\
\014\001\072\001\012\001\091\001\092\001\006\001\003\001\004\001\
\005\001\006\001\007\001\008\001\009\001\010\001\011\001\012\001\
\040\001\014\001\000\000\030\001\007\001\018\001\019\001\020\001\
\021\001\022\001\023\001\023\001\009\001\009\001\009\001\016\001\
\016\001\030\001\031\001\003\001\009\001\023\001\014\001\036\001\
\014\001\075\001\023\001\007\001\030\001\023\001\012\001\033\001\
\045\001\047\001\007\001\009\001\049\001\050\001\075\001\030\001\
\012\001\022\001\000\000\045\001\046\001\000\000\000\000\000\000\
\045\001\046\001\000\000\045\001\046\001\066\001\056\001\057\001\
\058\001\059\001\071\001\056\001\057\001\058\001\059\001\060\001\
\066\001\059\001\060\001\080\001\005\001\066\001\072\001\000\000\
\066\001\030\001\005\001\072\001\030\001\030\001\091\001\092\001\
\003\001\004\001\005\001\006\001\007\001\008\001\009\001\010\001\
\011\001\012\001\005\001\014\001\009\001\005\001\005\001\018\001\
\019\001\020\001\021\001\022\001\005\001\023\001\005\001\005\001\
\000\000\075\001\023\001\030\001\031\001\007\001\023\001\033\001\
\007\001\036\001\005\001\005\001\033\001\047\001\005\001\005\001\
\005\001\030\001\045\001\045\001\046\001\007\001\049\001\050\001\
\045\001\046\001\007\001\009\001\045\001\046\001\056\001\057\001\
\058\001\059\001\006\001\056\001\057\001\058\001\059\001\066\001\
\066\001\007\001\059\001\060\001\071\001\066\001\072\001\075\001\
\000\000\066\001\007\001\072\001\007\001\080\001\007\001\007\001\
\012\001\083\001\007\001\124\000\089\002\125\000\039\001\218\000\
\091\001\092\001\003\001\004\001\005\001\006\001\007\001\008\001\
\009\001\010\001\011\001\012\001\012\001\014\001\143\001\242\000\
\098\001\018\001\019\001\020\001\021\001\022\001\005\001\023\001\
\235\000\059\003\193\001\055\003\130\000\030\001\031\001\170\002\
\000\000\033\001\048\001\036\001\243\001\153\003\051\003\175\002\
\023\001\196\001\132\002\123\000\255\255\045\001\046\001\180\002\
\049\001\050\001\255\255\255\255\255\255\255\255\255\255\255\255\
\056\001\057\001\058\001\059\001\255\255\255\255\045\001\046\001\
\255\255\066\001\066\001\255\255\255\255\255\255\071\001\255\255\
\072\001\056\001\057\001\058\001\059\001\060\001\255\255\080\001\
\000\000\255\255\255\255\066\001\255\255\255\255\255\255\255\255\
\255\255\072\001\091\001\092\001\255\255\003\001\004\001\005\001\
\006\001\007\001\008\001\009\001\010\001\011\001\012\001\255\255\
\014\001\255\255\255\255\255\255\018\001\019\001\020\001\021\001\
\022\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\030\001\031\001\255\255\255\255\255\255\255\255\036\001\255\255\
\000\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\049\001\050\001\003\001\004\001\005\001\
\006\001\007\001\008\001\009\001\010\001\011\001\012\001\255\255\
\014\001\255\255\255\255\255\255\018\001\019\001\020\001\021\001\
\022\001\071\001\255\255\255\255\255\255\255\255\255\255\255\255\
\030\001\031\001\080\001\255\255\255\255\255\255\036\001\255\255\
\000\000\255\255\255\255\255\255\255\255\091\001\092\001\255\255\
\255\255\255\255\255\255\049\001\050\001\255\255\255\255\255\255\
\255\255\255\255\255\255\003\001\004\001\005\001\006\001\007\001\
\008\001\009\001\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\071\001\018\001\019\001\020\001\021\001\022\001\255\255\
\255\255\255\255\080\001\255\255\255\255\255\255\030\001\031\001\
\000\000\255\255\255\255\255\255\036\001\091\001\092\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\049\001\050\001\003\001\004\001\005\001\006\001\007\001\
\008\001\009\001\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\021\001\022\001\071\001\
\255\255\255\255\255\255\255\255\255\255\255\255\030\001\031\001\
\080\001\255\255\255\255\255\255\036\001\255\255\000\000\255\255\
\255\255\255\255\255\255\091\001\092\001\255\255\255\255\255\255\
\255\255\049\001\050\001\003\001\004\001\005\001\006\001\007\001\
\008\001\009\001\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\021\001\022\001\071\001\
\255\255\255\255\255\255\255\255\255\255\255\255\030\001\031\001\
\080\001\255\255\255\255\255\255\036\001\000\000\255\255\255\255\
\255\255\255\255\255\255\091\001\092\001\255\255\255\255\255\255\
\255\255\049\001\050\001\003\001\004\001\005\001\006\001\007\001\
\008\001\009\001\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\021\001\022\001\071\001\
\255\255\255\255\255\255\255\255\255\255\255\255\030\001\031\001\
\080\001\255\255\255\255\255\255\036\001\000\000\255\255\255\255\
\255\255\255\255\255\255\091\001\092\001\255\255\255\255\255\255\
\255\255\049\001\050\001\003\001\004\001\005\001\006\001\007\001\
\008\001\009\001\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\021\001\022\001\071\001\
\255\255\255\255\255\255\255\255\255\255\255\255\030\001\031\001\
\080\001\255\255\255\255\000\000\036\001\255\255\255\255\255\255\
\255\255\255\255\255\255\091\001\092\001\255\255\255\255\255\255\
\255\255\049\001\050\001\003\001\004\001\005\001\006\001\007\001\
\008\001\009\001\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\021\001\255\255\071\001\
\255\255\255\255\255\255\255\255\255\255\255\255\030\001\031\001\
\080\001\255\255\000\000\255\255\036\001\255\255\255\255\255\255\
\255\255\255\255\255\255\091\001\092\001\255\255\255\255\255\255\
\255\255\049\001\050\001\255\255\004\001\005\001\006\001\255\255\
\008\001\255\255\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\023\001\018\001\019\001\020\001\255\255\022\001\071\001\
\030\001\255\255\255\255\255\255\255\255\255\255\030\001\031\001\
\080\001\000\000\255\255\255\255\036\001\255\255\255\255\045\001\
\046\001\047\001\255\255\091\001\092\001\255\255\255\255\255\255\
\255\255\049\001\050\001\255\255\255\255\059\001\060\001\255\255\
\255\255\255\255\004\001\005\001\066\001\255\255\008\001\009\001\
\010\001\011\001\012\001\255\255\014\001\255\255\255\255\071\001\
\018\001\019\001\020\001\255\255\022\001\255\255\255\255\255\255\
\080\001\000\000\255\255\255\255\030\001\031\001\255\255\045\001\
\046\001\255\255\036\001\091\001\092\001\255\255\255\255\255\255\
\255\255\255\255\056\001\057\001\058\001\059\001\060\001\049\001\
\050\001\004\001\005\001\023\001\066\001\008\001\009\001\010\001\
\011\001\012\001\072\001\014\001\255\255\255\255\255\255\018\001\
\019\001\020\001\255\255\022\001\255\255\071\001\255\255\255\255\
\000\000\045\001\046\001\030\001\031\001\255\255\080\001\255\255\
\255\255\036\001\255\255\255\255\056\001\057\001\058\001\059\001\
\060\001\091\001\092\001\255\255\255\255\255\255\049\001\050\001\
\255\255\004\001\005\001\006\001\072\001\008\001\255\255\010\001\
\011\001\012\001\255\255\014\001\255\255\023\001\255\255\018\001\
\019\001\020\001\255\255\022\001\071\001\255\255\255\255\000\000\
\255\255\255\255\255\255\030\001\031\001\080\001\255\255\255\255\
\255\255\036\001\255\255\045\001\046\001\255\255\255\255\255\255\
\091\001\092\001\255\255\255\255\255\255\255\255\049\001\050\001\
\005\001\059\001\060\001\008\001\009\001\010\001\011\001\012\001\
\066\001\014\001\255\255\255\255\255\255\018\001\019\001\020\001\
\255\255\022\001\255\255\255\255\071\001\255\255\000\000\255\255\
\255\255\030\001\031\001\255\255\255\255\080\001\255\255\036\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\091\001\092\001\255\255\255\255\049\001\050\001\004\001\005\001\
\255\255\255\255\008\001\255\255\010\001\011\001\012\001\255\255\
\014\001\255\255\255\255\255\255\018\001\019\001\020\001\255\255\
\022\001\255\255\071\001\255\255\255\255\000\000\255\255\255\255\
\030\001\031\001\255\255\080\001\255\255\255\255\036\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\091\001\092\001\
\255\255\255\255\255\255\049\001\050\001\004\001\005\001\255\255\
\255\255\008\001\255\255\010\001\011\001\012\001\255\255\014\001\
\255\255\255\255\255\255\018\001\019\001\020\001\255\255\022\001\
\255\255\071\001\255\255\255\255\000\000\255\255\255\255\030\001\
\031\001\255\255\080\001\255\255\255\255\036\001\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\091\001\092\001\255\255\
\255\255\255\255\049\001\050\001\255\255\004\001\005\001\255\255\
\255\255\008\001\255\255\010\001\011\001\012\001\255\255\014\001\
\255\255\255\255\255\255\018\001\019\001\020\001\255\255\022\001\
\071\001\255\255\255\255\000\000\255\255\255\255\255\255\030\001\
\031\001\080\001\255\255\255\255\255\255\036\001\255\255\255\255\
\255\255\255\255\255\255\255\255\091\001\092\001\255\255\255\255\
\255\255\255\255\049\001\050\001\004\001\005\001\255\255\255\255\
\255\255\255\255\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\255\255\022\001\255\255\
\071\001\255\255\000\000\255\255\255\255\255\255\030\001\031\001\
\255\255\080\001\255\255\255\255\036\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\091\001\092\001\255\255\255\255\
\255\255\049\001\050\001\004\001\005\001\255\255\255\255\255\255\
\255\255\010\001\011\001\012\001\255\255\014\001\255\255\255\255\
\255\255\018\001\019\001\020\001\255\255\022\001\255\255\071\001\
\255\255\000\000\255\255\255\255\255\255\030\001\031\001\255\255\
\080\001\255\255\255\255\036\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\091\001\092\001\255\255\255\255\255\255\
\049\001\050\001\004\001\005\001\255\255\255\255\008\001\255\255\
\010\001\011\001\255\255\255\255\014\001\255\255\255\255\255\255\
\018\001\019\001\020\001\255\255\022\001\255\255\071\001\255\255\
\000\000\255\255\255\255\255\255\030\001\031\001\255\255\080\001\
\255\255\255\255\036\001\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\091\001\092\001\255\255\255\255\255\255\049\001\
\050\001\004\001\005\001\255\255\255\255\255\255\255\255\010\001\
\011\001\012\001\255\255\014\001\255\255\255\255\255\255\018\001\
\019\001\020\001\255\255\022\001\255\255\071\001\255\255\000\000\
\255\255\255\255\255\255\030\001\031\001\255\255\080\001\255\255\
\255\255\036\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\091\001\092\001\255\255\255\255\255\255\049\001\050\001\
\004\001\005\001\255\255\255\255\008\001\255\255\010\001\011\001\
\255\255\255\255\014\001\255\255\255\255\255\255\018\001\019\001\
\020\001\255\255\022\001\255\255\071\001\255\255\255\255\255\255\
\255\255\255\255\030\001\031\001\000\000\080\001\255\255\255\255\
\036\001\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
\091\001\092\001\255\255\255\255\255\255\049\001\050\001\004\001\
\005\001\255\255\255\255\255\255\255\255\010\001\011\001\012\001\
\255\255\014\001\255\255\255\255\255\255\018\001\019\001\020\001\
\255\255\022\001\255\255\071\001\255\255\255\255\255\255\255\255\
\255\255\030\001\031\001\255\255\080\001\255\255\255\255\036\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\091\001\
\092\001\255\255\255\255\255\255\049\001\050\001\004\001\005\001\
\255\255\255\255\008\001\255\255\010\001\011\001\255\255\255\255\
\014\001\255\255\255\255\255\255\018\001\019\001\020\001\255\255\
\022\001\255\255\071\001\255\255\255\255\255\255\255\255\255\255\
\030\001\031\001\255\255\080\001\255\255\255\255\036\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\091\001\092\001\
\255\255\255\255\255\255\049\001\050\001\004\001\005\001\255\255\
\255\255\008\001\255\255\010\001\011\001\012\001\255\255\014\001\
\255\255\255\255\255\255\018\001\019\001\020\001\255\255\255\255\
\255\255\071\001\255\255\255\255\255\255\255\255\255\255\030\001\
\031\001\255\255\080\001\255\255\255\255\036\001\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\091\001\092\001\255\255\
\255\255\255\255\049\001\050\001\004\001\005\001\255\255\255\255\
\255\255\255\255\010\001\011\001\012\001\255\255\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\255\255\022\001\255\255\
\071\001\255\255\255\255\255\255\255\255\255\255\030\001\031\001\
\255\255\080\001\255\255\255\255\036\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\091\001\092\001\255\255\255\255\
\255\255\049\001\050\001\004\001\005\001\255\255\255\255\255\255\
\255\255\010\001\011\001\012\001\255\255\014\001\255\255\255\255\
\255\255\018\001\019\001\020\001\255\255\255\255\255\255\071\001\
\255\255\255\255\255\255\255\255\255\255\030\001\031\001\255\255\
\080\001\255\255\255\255\036\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\091\001\092\001\255\255\255\255\255\255\
\049\001\050\001\255\255\255\255\255\255\255\255\255\255\003\001\
\255\255\005\001\006\001\007\001\003\001\009\001\005\001\006\001\
\007\001\255\255\009\001\255\255\255\255\255\255\071\001\255\255\
\255\255\255\255\255\255\023\001\255\255\255\255\255\255\080\001\
\023\001\255\255\030\001\255\255\255\255\255\255\023\001\030\001\
\255\255\255\255\091\001\092\001\255\255\255\255\042\001\043\001\
\044\001\045\001\046\001\042\001\043\001\044\001\045\001\046\001\
\255\255\255\255\255\255\255\255\045\001\046\001\255\255\059\001\
\060\001\255\255\255\255\255\255\255\255\060\001\066\001\056\001\
\057\001\058\001\059\001\066\001\255\255\255\255\255\255\255\255\
\255\255\066\001\255\255\079\001\255\255\004\001\255\255\072\001\
\079\001\008\001\255\255\010\001\011\001\012\001\255\255\014\001\
\255\255\255\255\255\255\018\001\019\001\020\001\255\255\255\255\
\255\255\255\255\255\255\255\255\004\001\255\255\255\255\255\255\
\031\001\003\001\010\001\011\001\012\001\036\001\014\001\255\255\
\255\255\255\255\018\001\019\001\020\001\255\255\255\255\255\255\
\255\255\255\255\049\001\050\001\255\255\023\001\255\255\031\001\
\255\255\255\255\255\255\255\255\036\001\255\255\255\255\033\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\071\001\049\001\050\001\045\001\046\001\255\255\255\255\255\255\
\255\255\080\001\255\255\255\255\255\255\255\255\056\001\057\001\
\058\001\059\001\255\255\004\001\091\001\092\001\255\255\071\001\
\066\001\010\001\011\001\012\001\255\255\014\001\072\001\255\255\
\080\001\018\001\019\001\020\001\003\001\255\255\005\001\006\001\
\007\001\255\255\009\001\091\001\092\001\012\001\031\001\255\255\
\255\255\255\255\255\255\036\001\255\255\255\255\255\255\255\255\
\023\001\255\255\255\255\255\255\255\255\255\255\255\255\030\001\
\049\001\050\001\033\001\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\045\001\046\001\
\255\255\255\255\255\255\255\255\255\255\255\255\071\001\255\255\
\255\255\056\001\057\001\058\001\059\001\255\255\003\001\080\001\
\005\001\006\001\007\001\066\001\009\001\255\255\255\255\012\001\
\255\255\072\001\091\001\092\001\255\255\255\255\023\001\255\255\
\255\255\255\255\023\001\255\255\083\001\030\001\255\255\255\255\
\033\001\030\001\089\001\255\255\033\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\045\001\046\001\255\255\255\255\
\045\001\046\001\255\255\255\255\255\255\255\255\255\255\056\001\
\057\001\058\001\059\001\056\001\057\001\058\001\059\001\255\255\
\003\001\066\001\005\001\006\001\007\001\066\001\009\001\072\001\
\255\255\012\001\255\255\072\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\023\001\255\255\083\001\255\255\
\255\255\255\255\255\255\030\001\089\001\003\001\033\001\005\001\
\255\255\007\001\255\255\009\001\255\255\255\255\012\001\255\255\
\255\255\255\255\045\001\046\001\255\255\255\255\255\255\255\255\
\255\255\023\001\255\255\255\255\255\255\056\001\057\001\058\001\
\030\001\255\255\255\255\033\001\255\255\255\255\255\255\066\001\
\255\255\255\255\255\255\255\255\255\255\072\001\255\255\045\001\
\046\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\083\001\255\255\056\001\057\001\058\001\059\001\089\001\003\001\
\255\255\005\001\006\001\007\001\066\001\009\001\255\255\255\255\
\012\001\255\255\072\001\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\023\001\255\255\083\001\255\255\255\255\
\255\255\255\255\030\001\089\001\003\001\033\001\005\001\006\001\
\007\001\255\255\009\001\255\255\255\255\012\001\255\255\255\255\
\255\255\045\001\046\001\255\255\255\255\255\255\255\255\255\255\
\023\001\255\255\255\255\255\255\255\255\057\001\058\001\030\001\
\255\255\003\001\033\001\005\001\006\001\007\001\066\001\009\001\
\255\255\255\255\012\001\255\255\072\001\255\255\045\001\046\001\
\255\255\255\255\255\255\255\255\255\255\023\001\255\255\083\001\
\255\255\255\255\255\255\058\001\030\001\089\001\003\001\033\001\
\005\001\006\001\007\001\066\001\009\001\255\255\255\255\012\001\
\255\255\072\001\255\255\045\001\046\001\255\255\255\255\255\255\
\255\255\255\255\023\001\255\255\083\001\255\255\255\255\255\255\
\255\255\030\001\089\001\003\001\033\001\005\001\006\001\007\001\
\066\001\009\001\255\255\255\255\012\001\255\255\072\001\255\255\
\045\001\046\001\255\255\255\255\255\255\255\255\255\255\023\001\
\255\255\083\001\255\255\255\255\255\255\255\255\030\001\089\001\
\003\001\033\001\005\001\006\001\007\001\066\001\009\001\255\255\
\006\001\012\001\255\255\009\001\255\255\045\001\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\083\001\255\255\
\255\255\023\001\255\255\030\001\089\001\255\255\033\001\255\255\
\030\001\255\255\066\001\033\001\255\255\255\255\255\255\255\255\
\255\255\255\255\045\001\255\255\255\255\003\001\255\255\045\001\
\046\001\255\255\003\001\083\001\255\255\255\255\255\255\255\255\
\255\255\089\001\056\001\057\001\058\001\059\001\255\255\066\001\
\255\255\023\001\255\255\255\255\066\001\003\001\023\001\255\255\
\255\255\255\255\072\001\033\001\255\255\255\255\255\255\255\255\
\083\001\255\255\255\255\255\255\255\255\083\001\089\001\045\001\
\046\001\023\001\255\255\089\001\045\001\046\001\255\255\255\255\
\255\255\003\001\056\001\057\001\058\001\059\001\255\255\056\001\
\057\001\058\001\059\001\060\001\066\001\255\255\255\255\045\001\
\046\001\066\001\072\001\003\001\255\255\023\001\255\255\072\001\
\255\255\255\255\056\001\057\001\058\001\059\001\060\001\255\255\
\255\255\255\255\255\255\255\255\066\001\255\255\255\255\023\001\
\255\255\255\255\072\001\045\001\046\001\005\001\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\056\001\057\001\
\058\001\059\001\060\001\255\255\255\255\045\001\046\001\023\001\
\066\001\255\255\255\255\255\255\255\255\255\255\072\001\255\255\
\056\001\057\001\058\001\059\001\060\001\255\255\255\255\255\255\
\255\255\255\255\066\001\255\255\255\255\045\001\046\001\255\255\
\072\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\056\001\057\001\058\001\059\001\060\001\255\255\024\001\025\001\
\026\001\027\001\066\001\255\255\255\255\031\001\032\001\255\255\
\072\001\035\001\036\001\037\001\038\001\039\001\255\255\041\001\
\255\255\255\255\009\001\255\255\255\255\255\255\048\001\255\255\
\255\255\255\255\052\001\053\001\255\255\255\255\255\255\255\255\
\023\001\255\255\255\255\061\001\062\001\255\255\064\001\255\255\
\255\255\067\001\068\001\069\001\070\001\255\255\255\255\073\001\
\255\255\009\001\076\001\077\001\255\255\255\255\045\001\046\001\
\255\255\255\255\255\255\255\255\086\001\087\001\088\001\023\001\
\090\001\056\001\057\001\058\001\059\001\060\001\255\255\012\001\
\255\255\033\001\255\255\066\001\012\001\255\255\255\255\255\255\
\255\255\072\001\023\001\255\255\255\255\045\001\046\001\023\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\056\001\057\001\058\001\059\001\012\001\255\255\255\255\255\255\
\045\001\046\001\066\001\255\255\255\255\045\001\046\001\023\001\
\072\001\255\255\255\255\056\001\057\001\058\001\059\001\060\001\
\056\001\057\001\058\001\059\001\060\001\066\001\021\001\255\255\
\023\001\255\255\066\001\072\001\255\255\045\001\046\001\255\255\
\072\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\056\001\057\001\058\001\059\001\060\001\255\255\045\001\046\001\
\255\255\255\255\066\001\021\001\255\255\023\001\255\255\255\255\
\072\001\056\001\057\001\058\001\059\001\060\001\255\255\255\255\
\255\255\255\255\255\255\066\001\255\255\022\001\023\001\255\255\
\255\255\072\001\255\255\045\001\046\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\056\001\057\001\
\058\001\059\001\060\001\255\255\045\001\046\001\023\001\255\255\
\066\001\255\255\255\255\255\255\255\255\030\001\072\001\056\001\
\057\001\058\001\059\001\060\001\255\255\255\255\255\255\023\001\
\255\255\066\001\255\255\255\255\045\001\046\001\030\001\072\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\056\001\
\057\001\058\001\059\001\060\001\255\255\045\001\046\001\023\001\
\255\255\066\001\255\255\255\255\023\001\255\255\255\255\072\001\
\056\001\057\001\058\001\059\001\060\001\255\255\033\001\255\255\
\255\255\255\255\066\001\255\255\255\255\045\001\046\001\255\255\
\072\001\255\255\045\001\046\001\255\255\255\255\255\255\255\255\
\056\001\057\001\058\001\059\001\060\001\056\001\057\001\058\001\
\059\001\255\255\066\001\255\255\255\255\255\255\255\255\066\001\
\072\001\255\255\255\255\255\255\255\255\072\001"

let yynames_const = "\
  CHOICE\000\
  STAR\000\
  COMMA\000\
  LPAREN\000\
  RPAREN\000\
  LBRACKET\000\
  RBRACKET\000\
  BAR\000\
  SEMI\000\
  NEW\000\
  OUT\000\
  IN\000\
  AT\000\
  REPL\000\
  IF\000\
  THEN\000\
  ELSE\000\
  EQUAL\000\
  FUN\000\
  EQUATION\000\
  REDUCTION\000\
  PREDICATE\000\
  PROCESS\000\
  SLASH\000\
  DOT\000\
  EOF\000\
  LET\000\
  QUERY\000\
  BEFORE\000\
  PUTBEGIN\000\
  NONINTERF\000\
  EVENT\000\
  NOT\000\
  ELIMTRUE\000\
  FREE\000\
  SUCHTHAT\000\
  CLAUSES\000\
  RED\000\
  EQUIV\000\
  EQUIVEQ\000\
  WEDGE\000\
  DIFF\000\
  COLON\000\
  NOUNIF\000\
  PHASE\000\
  BARRIER\000\
  AMONG\000\
  WEAKSECRET\000\
  PARAM\000\
  ORTEXT\000\
  FAIL\000\
  LESS\000\
  GREATER\000\
  GEQ\000\
  PLUS\000\
  MINUS\000\
  TYPE\000\
  SET\000\
  FORALL\000\
  CONST\000\
  INJEVENT\000\
  OR\000\
  CHANNEL\000\
  LETFUN\000\
  DEFINE\000\
  EXPAND\000\
  YIELD\000\
  LEQ\000\
  PROBA\000\
  LBRACE\000\
  RBRACE\000\
  PROOF\000\
  IMPLEMENTATION\000\
  EQUIVALENCE\000\
  OTHERWISE\000\
  FOREACH\000\
  DO\000\
  SECRET\000\
  PUBLICVARS\000\
  RANDOM\000\
  LEFTARROW\000\
  LEMMA\000\
  AXIOM\000\
  RESTRICTION\000\
  FOR\000\
  TABLE\000\
  INSERT\000\
  GET\000\
  "

let yynames_block = "\
  IDENT\000\
  TAG\000\
  STRING\000\
  PROJECTION\000\
  INT\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 206 "pitparser.mly"
      ( (* Options are ignored, they are supported for compatibility with
             CryptoVerif only *)
          TTypeDecl(_2) :: _5 )
# 1852 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 8 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 6 : 'typeidseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'typeid) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _10 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 210 "pitparser.mly"
      ( (TFunDecl(_2, _4, _7, _8)) :: _10 )
# 1863 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 10 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 8 : 'typeidseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 5 : 'typeid) in
    let _9 = (Parsing.peek_val __caml_parser_env 3 : 'treducmayfail) in
    let _10 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _12 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 212 "pitparser.mly"
      ( (TReducFail(_2,_4,_7,_9, _10)) :: _12 )
# 1875 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'treduc) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 214 "pitparser.mly"
      ( (TReduc(_2,_3)) :: _5 )
# 1884 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'neidentseq) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'typeid) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 216 "pitparser.mly"
      ( (List.map (fun x -> TConstDecl(x, _4, _5)) _2) @ _7 )
# 1894 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'eqlist) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 218 "pitparser.mly"
      ( (TEquation(_2, _3)) :: _5 )
# 1903 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 220 "pitparser.mly"
      ( (TEventDecl(_2, [])) :: _4 )
# 1911 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'typeidseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 222 "pitparser.mly"
      ( (TEventDecl(_2, _4)) :: _7 )
# 1920 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 6 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 4 : 'typeidseq) in
    let _6 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _8 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 224 "pitparser.mly"
      ( (TPredDecl(_2, _4, _6)) :: _8 )
# 1930 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 226 "pitparser.mly"
      ( (TPredDecl(_2, [], _3)) :: _5 )
# 1939 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'typeidseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 228 "pitparser.mly"
      ( (TTableDecl(_2, _4)) :: _7 )
# 1948 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'tprocess) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 230 "pitparser.mly"
      ( (TPDef(_2,[],_4)) :: _6 )
# 1957 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 7 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 5 : 'mayfailvartypeseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : 'tprocess) in
    let _9 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 232 "pitparser.mly"
      ( (TPDef(_2,_4,_7)) :: _9 )
# 1967 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 234 "pitparser.mly"
      ( (TLetFun(_2,[],_4)) :: _6 )
# 1976 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 7 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 5 : 'mayfailvartypeseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _9 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 236 "pitparser.mly"
      ( (TLetFun(_2,_4,_7)) :: _9 )
# 1986 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 238 "pitparser.mly"
      ( (TSet(_2,S _4)) :: _6 )
# 1995 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 240 "pitparser.mly"
      ( (TSet(_2,S _4)) :: _6 )
# 2004 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : int) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 242 "pitparser.mly"
      ( (TSet(_2,I _4)) :: _6 )
# 2013 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'tfnebindingseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 244 "pitparser.mly"
      ( (TNoUnif (_2, _4, _5)) :: _7 )
# 2023 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'tfnebindingseq) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 246 "pitparser.mly"
      ( (TNoUnif ([], _2, _3)) :: _5 )
# 2032 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'tqueryseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 248 "pitparser.mly"
      ( (TQuery(_2,_4,_5)) :: _7 )
# 2042 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'tqueryseq) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 250 "pitparser.mly"
      ( (TQuery([],_2,_3)) :: _5 )
# 2051 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'niseq) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 252 "pitparser.mly"
      ( (TNoninterf(_2, _4)) :: _6 )
# 2060 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'niseq) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 254 "pitparser.mly"
      ( (TNoninterf([], _2)) :: _4 )
# 2068 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 256 "pitparser.mly"
      ( (TWeaksecret(_2)) :: _4 )
# 2076 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 258 "pitparser.mly"
      ( (TNot(_2, _4)) :: _6 )
# 2085 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 260 "pitparser.mly"
      ( (TNot([], _2)) :: _4 )
# 2093 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'neidentseq) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 262 "pitparser.mly"
      ( (* Supported for compatility with CryptoVerif only *)
        _5 )
# 2103 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 265 "pitparser.mly"
      ( (* Supported for compatility with CryptoVerif only *)
        _4 )
# 2112 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'proof) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 268 "pitparser.mly"
      ( (* Supported for compatility with CryptoVerif only *)
        _5 )
# 2121 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'impllist) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 271 "pitparser.mly"
      ( (* Supported for compatility with CryptoVerif only *)
        _4 )
# 2130 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : 'nemayfailvartypeseq) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 274 "pitparser.mly"
      ( (TElimtrue (_2,_4)) :: _6 )
# 2139 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 276 "pitparser.mly"
      ( (TElimtrue ([],_2)) :: _4 )
# 2147 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'neidentseq) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 278 "pitparser.mly"
      ( (* For compatibility with CryptoVerif, allow
            channel c1...cn.
           as a synonym for
            free c1...cn:channel. *)
        (List.map (fun x -> TFree(x, ("channel", dummy_ext), [])) _2) @ _4 )
# 2159 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'neidentseq) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'typeid) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 284 "pitparser.mly"
      ( (List.map (fun x -> TFree(x, _4, _5)) _2) @ _7 )
# 2169 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'tclauses) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 286 "pitparser.mly"
      ( (TClauses(_2)) :: _3 )
# 2177 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 7 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 5 : 'typeidseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.tdecl list) in
    let _9 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 288 "pitparser.mly"
      ( (TDefine(_2, _4, _7)) :: _9 )
# 2187 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'typeidseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 290 "pitparser.mly"
      ( (TExpand(_2, _4)) :: _7 )
# 2196 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 6 : 'lemma) in
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'tlemmaseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 292 "pitparser.mly"
      ( (TLemma(_1,_2,_4,_5)) :: _7 )
# 2207 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'lemma) in
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'tlemmaseq) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.tdecl list) in
    Obj.repr(
# 294 "pitparser.mly"
      ( (TLemma(_1,[],_2,_3)) :: _5 )
# 2217 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    Obj.repr(
# 296 "pitparser.mly"
      ( [] )
# 2223 "pitparser.ml"
               : Pitptree.tdecl list))
; (fun __caml_parser_env ->
    Obj.repr(
# 300 "pitparser.mly"
      ( KLemma )
# 2229 "pitparser.ml"
               : 'lemma))
; (fun __caml_parser_env ->
    Obj.repr(
# 302 "pitparser.mly"
      ( KAxiom )
# 2235 "pitparser.ml"
               : 'lemma))
; (fun __caml_parser_env ->
    Obj.repr(
# 304 "pitparser.mly"
      ( KRestriction )
# 2241 "pitparser.ml"
               : 'lemma))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.tdecl list) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'tprocess) in
    Obj.repr(
# 308 "pitparser.mly"
 ( _1, _3, None )
# 2249 "pitparser.ml"
               : Pitptree.tdecl list * Pitptree.tprocess_e * Pitptree.tprocess_e option))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.tdecl list) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'tprocess) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'tprocess) in
    Obj.repr(
# 310 "pitparser.mly"
 ( _1, _3, Some _4 )
# 2258 "pitparser.ml"
               : Pitptree.tdecl list * Pitptree.tprocess_e * Pitptree.tprocess_e option))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 316 "pitparser.mly"
        ( _1 )
# 2265 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 318 "pitparser.mly"
        ( _1 )
# 2272 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 320 "pitparser.mly"
        ( string_of_int _1, parse_extent() )
# 2279 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 322 "pitparser.mly"
        ( "*", parse_extent() )
# 2285 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 324 "pitparser.mly"
        ( ".", parse_extent() )
# 2291 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 326 "pitparser.mly"
        ( "set", parse_extent() )
# 2297 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 328 "pitparser.mly"
        ( "insert", parse_extent() )
# 2303 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 330 "pitparser.mly"
        ( "=", parse_extent() )
# 2309 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 332 "pitparser.mly"
        ( ",", parse_extent() )
# 2315 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 334 "pitparser.mly"
        ( "(", parse_extent() )
# 2321 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    Obj.repr(
# 336 "pitparser.mly"
        ( ")", parse_extent() )
# 2327 "pitparser.ml"
               : 'prooftoken))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'prooftoken) in
    Obj.repr(
# 340 "pitparser.mly"
        ( [_1] )
# 2334 "pitparser.ml"
               : 'proofcommand))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'prooftoken) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'proofcommand) in
    Obj.repr(
# 342 "pitparser.mly"
        ( _1 :: _2 )
# 2342 "pitparser.ml"
               : 'proofcommand))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'proofcommand) in
    Obj.repr(
# 346 "pitparser.mly"
 ( [_1] )
# 2349 "pitparser.ml"
               : 'proof))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'proofcommand) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'proof) in
    Obj.repr(
# 348 "pitparser.mly"
        ( _1 :: _3 )
# 2357 "pitparser.ml"
               : 'proof))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'impl) in
    Obj.repr(
# 354 "pitparser.mly"
        ( [_1] )
# 2364 "pitparser.ml"
               : 'impllist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'impl) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'impllist) in
    Obj.repr(
# 356 "pitparser.mly"
        ( _1 :: _3 )
# 2372 "pitparser.ml"
               : 'impllist))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'cvtypeid) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'typeoptions) in
    Obj.repr(
# 360 "pitparser.mly"
        ( () )
# 2381 "pitparser.ml"
               : 'impl))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.ident) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'functionoptions) in
    Obj.repr(
# 362 "pitparser.mly"
        ( () )
# 2390 "pitparser.ml"
               : 'impl))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 364 "pitparser.mly"
        ( () )
# 2398 "pitparser.ml"
               : 'impl))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 366 "pitparser.mly"
        ( () )
# 2406 "pitparser.ml"
               : 'impl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 370 "pitparser.mly"
        ( () )
# 2413 "pitparser.ml"
               : 'cvtypeid))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 372 "pitparser.mly"
        ( () )
# 2420 "pitparser.ml"
               : 'cvtypeid))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 376 "pitparser.mly"
        ( [_1] )
# 2427 "pitparser.ml"
               : 'stringlistne))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'stringlistne) in
    Obj.repr(
# 378 "pitparser.mly"
        ( _1::_3 )
# 2435 "pitparser.ml"
               : 'stringlistne))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'stringlistne) in
    Obj.repr(
# 382 "pitparser.mly"
        ( _1,_3 )
# 2443 "pitparser.ml"
               : 'typeopt))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'stringlistne) in
    Obj.repr(
# 384 "pitparser.mly"
        ( ("pred",parse_extent()),_3 )
# 2450 "pitparser.ml"
               : 'typeopt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'typeopt) in
    Obj.repr(
# 388 "pitparser.mly"
        ( [_1] )
# 2457 "pitparser.ml"
               : 'typeoptlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'typeopt) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'typeoptlist) in
    Obj.repr(
# 390 "pitparser.mly"
        ( _1::_3 )
# 2465 "pitparser.ml"
               : 'typeoptlist))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'typeoptlist) in
    Obj.repr(
# 394 "pitparser.mly"
        ( _2 )
# 2472 "pitparser.ml"
               : 'typeoptions))
; (fun __caml_parser_env ->
    Obj.repr(
# 396 "pitparser.mly"
        ( [] )
# 2478 "pitparser.ml"
               : 'typeoptions))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 400 "pitparser.mly"
        ( _1,_3 )
# 2486 "pitparser.ml"
               : 'funopt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'funopt) in
    Obj.repr(
# 404 "pitparser.mly"
        ( [_1] )
# 2493 "pitparser.ml"
               : 'funoptlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'funopt) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'funoptlist) in
    Obj.repr(
# 406 "pitparser.mly"
        ( _1::_3 )
# 2501 "pitparser.ml"
               : 'funoptlist))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'funoptlist) in
    Obj.repr(
# 410 "pitparser.mly"
        ( _2 )
# 2508 "pitparser.ml"
               : 'functionoptions))
; (fun __caml_parser_env ->
    Obj.repr(
# 412 "pitparser.mly"
        ( [] )
# 2514 "pitparser.ml"
               : 'functionoptions))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'optionseq) in
    Obj.repr(
# 418 "pitparser.mly"
      ( _2 )
# 2521 "pitparser.ml"
               : 'options))
; (fun __caml_parser_env ->
    Obj.repr(
# 420 "pitparser.mly"
      ( [] )
# 2527 "pitparser.ml"
               : 'options))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 424 "pitparser.mly"
      ( _1, None )
# 2534 "pitparser.ml"
               : 'singleoption))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 426 "pitparser.mly"
      ( _1, Some [_3] )
# 2542 "pitparser.ml"
               : 'singleoption))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'neidentseq) in
    Obj.repr(
# 428 "pitparser.mly"
      ( _1, Some _4 )
# 2550 "pitparser.ml"
               : 'singleoption))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'singleoption) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'optionseq) in
    Obj.repr(
# 432 "pitparser.mly"
      ( _1 :: _3 )
# 2558 "pitparser.ml"
               : 'optionseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'singleoption) in
    Obj.repr(
# 434 "pitparser.mly"
      ( [_1] )
# 2565 "pitparser.ml"
               : 'optionseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'neidentseq) in
    Obj.repr(
# 438 "pitparser.mly"
    ( _1 :: _3 )
# 2573 "pitparser.ml"
               : 'neidentseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 440 "pitparser.mly"
    ( [_1] )
# 2580 "pitparser.ml"
               : 'neidentseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 444 "pitparser.mly"
    ( None )
# 2586 "pitparser.ml"
               : 'newarg))
; (fun __caml_parser_env ->
    Obj.repr(
# 446 "pitparser.mly"
    ( Some [] )
# 2592 "pitparser.ml"
               : 'newarg))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'neidentseq) in
    Obj.repr(
# 448 "pitparser.mly"
    ( Some (_2) )
# 2599 "pitparser.ml"
               : 'newarg))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'neidentseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'typeid) in
    Obj.repr(
# 455 "pitparser.mly"
    ( List.map (fun x -> (x,_5)) (_1::_3) )
# 2608 "pitparser.ml"
               : 'onevartype))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'typeid) in
    Obj.repr(
# 457 "pitparser.mly"
    ( [(_1,_3)] )
# 2616 "pitparser.ml"
               : 'onevartype))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'onevartype) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'nevartype) in
    Obj.repr(
# 461 "pitparser.mly"
        ( _1 @ _3 )
# 2624 "pitparser.ml"
               : 'nevartype))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'onevartype) in
    Obj.repr(
# 464 "pitparser.mly"
        ( _1 )
# 2631 "pitparser.ml"
               : 'nevartype))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'nevartype) in
    Obj.repr(
# 468 "pitparser.mly"
        ( _2 )
# 2638 "pitparser.ml"
               : 'forallvartype))
; (fun __caml_parser_env ->
    Obj.repr(
# 470 "pitparser.mly"
        ( [] )
# 2644 "pitparser.ml"
               : 'forallvartype))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 474 "pitparser.mly"
        ( _1 )
# 2651 "pitparser.ml"
               : 'typeid))
; (fun __caml_parser_env ->
    Obj.repr(
# 476 "pitparser.mly"
        ( (* channel is allowed as a type, even though it is also a keyword for the declaration channel c1...cn. *)
          "channel", parse_extent() )
# 2658 "pitparser.ml"
               : 'typeid))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'netypeidseq) in
    Obj.repr(
# 481 "pitparser.mly"
        ( _1 )
# 2665 "pitparser.ml"
               : 'typeidseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 483 "pitparser.mly"
        ( [] )
# 2671 "pitparser.ml"
               : 'typeidseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'typeid) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'netypeidseq) in
    Obj.repr(
# 487 "pitparser.mly"
    ( _1 :: _3 )
# 2679 "pitparser.ml"
               : 'netypeidseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'typeid) in
    Obj.repr(
# 489 "pitparser.mly"
    ( [_1] )
# 2686 "pitparser.ml"
               : 'netypeidseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 495 "pitparser.mly"
 ( PFail, parse_extent () )
# 2692 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'termseq) in
    Obj.repr(
# 497 "pitparser.mly"
        ( PFunApp (_1, _3), parse_extent() )
# 2700 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.term_e) in
    Obj.repr(
# 499 "pitparser.mly"
        ( PProj (_1,_3), parse_extent() )
# 2708 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.term_e) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.term_e) in
    Obj.repr(
# 501 "pitparser.mly"
        ( Param.has_choice := true;
	  PFunApp(("choice", parse_extent()), [_3; _5]), parse_extent() )
# 2717 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 504 "pitparser.mly"
 ( PIdent (_1), parse_extent() )
# 2724 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 506 "pitparser.mly"
        ( Param.has_integer := true; unfold_int (zero ()) _1 )
# 2731 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 508 "pitparser.mly"
        ( Param.has_integer := true; unfold_int_minus _1 _3 )
# 2739 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 510 "pitparser.mly"
        ( Param.has_integer := true; unfold_int _1 _3 )
# 2747 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : int) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 512 "pitparser.mly"
        ( Param.has_integer := true; unfold_int _3 _1 )
# 2755 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 514 "pitparser.mly"
        ( PFunApp(("=", parse_extent()), [_1; _3]), parse_extent() )
# 2763 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 516 "pitparser.mly"
        ( PFunApp(("<>", parse_extent()), [_1; _3]), parse_extent() )
# 2771 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.term_e) in
    Obj.repr(
# 518 "pitparser.mly"
        ( PFunApp(("not", parse_extent()), [_3]), parse_extent() )
# 2778 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 520 "pitparser.mly"
        ( PFunApp(("||", parse_extent()), [_1; _3]), parse_extent() )
# 2786 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 522 "pitparser.mly"
        ( PFunApp(("&&", parse_extent()), [_1; _3]), parse_extent() )
# 2794 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'termseq) in
    Obj.repr(
# 524 "pitparser.mly"
 ( match _2 with
	  [t] -> t   (* Allow parentheses for priorities of infix operators;
			Tuples cannot have one element. *)
	| l -> PTuple (l), parse_extent() )
# 2804 "pitparser.ml"
               : Pitptree.term_e))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'netermseq) in
    Obj.repr(
# 531 "pitparser.mly"
 ( _1 :: _3 )
# 2812 "pitparser.ml"
               : 'netermseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 533 "pitparser.mly"
 ( [_1] )
# 2819 "pitparser.ml"
               : 'netermseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'netermseq) in
    Obj.repr(
# 537 "pitparser.mly"
        ( _1 )
# 2826 "pitparser.ml"
               : 'termseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 539 "pitparser.mly"
        ( [] )
# 2832 "pitparser.ml"
               : 'termseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 6 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'netermseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'niseq) in
    Obj.repr(
# 553 "pitparser.mly"
    ( (_1,Some _4) :: _7 )
# 2841 "pitparser.ml"
               : 'niseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'netermseq) in
    Obj.repr(
# 555 "pitparser.mly"
    ( [_1,Some _4] )
# 2849 "pitparser.ml"
               : 'niseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'niseq) in
    Obj.repr(
# 557 "pitparser.mly"
    ( (_1,None) :: _3 )
# 2857 "pitparser.ml"
               : 'niseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 559 "pitparser.mly"
    ( [_1,None] )
# 2864 "pitparser.ml"
               : 'niseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 564 "pitparser.mly"
    ( None, [] )
# 2870 "pitparser.ml"
               : 'opt_publivars_ror))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'neidentseq) in
    Obj.repr(
# 566 "pitparser.mly"
    ( None, _4 )
# 2877 "pitparser.ml"
               : 'opt_publivars_ror))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _5 = (Parsing.peek_val __caml_parser_env 4 : 'optpublicvars) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    Obj.repr(
# 568 "pitparser.mly"
    ( Some (_4,_7), _5 )
# 2886 "pitparser.ml"
               : 'opt_publivars_ror))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'gterm) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'opt_publivars_ror) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'tlemmaseq) in
    Obj.repr(
# 572 "pitparser.mly"
      ( let (ror,pubvars) = _2 in (_1,ror,pubvars) :: _4 )
# 2895 "pitparser.ml"
               : 'tlemmaseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'gterm) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'opt_publivars_ror) in
    Obj.repr(
# 574 "pitparser.mly"
      ( let (ror,pubvars) = _2 in [_1,ror,pubvars] )
# 2903 "pitparser.ml"
               : 'tlemmaseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'tquery) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'tqueryseq) in
    Obj.repr(
# 580 "pitparser.mly"
    ( _1 :: _3 )
# 2911 "pitparser.ml"
               : 'tqueryseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'tquery) in
    Obj.repr(
# 582 "pitparser.mly"
    ( [_1] )
# 2918 "pitparser.ml"
               : 'tqueryseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'gterm) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'optpublicvars) in
    Obj.repr(
# 586 "pitparser.mly"
    ( PRealQuery(_1,_2), parse_extent() )
# 2926 "pitparser.ml"
               : 'tquery))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'optpublicvars) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'options) in
    Obj.repr(
# 588 "pitparser.mly"
    ( PQSecret (_2,_3,_4), parse_extent() )
# 2935 "pitparser.ml"
               : 'tquery))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'neidentseq) in
    Obj.repr(
# 590 "pitparser.mly"
    ( PPutBegin(false, _4), parse_extent() )
# 2942 "pitparser.ml"
               : 'tquery))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'neidentseq) in
    Obj.repr(
# 592 "pitparser.mly"
    ( PPutBegin(true, _4), parse_extent() )
# 2949 "pitparser.ml"
               : 'tquery))
; (fun __caml_parser_env ->
    Obj.repr(
# 596 "pitparser.mly"
    ( [] )
# 2955 "pitparser.ml"
               : 'optpublicvars))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'neidentseq) in
    Obj.repr(
# 598 "pitparser.mly"
    ( _2 )
# 2962 "pitparser.ml"
               : 'optpublicvars))
; (fun __caml_parser_env ->
    Obj.repr(
# 602 "pitparser.mly"
    ( None )
# 2968 "pitparser.ml"
               : 'optatident))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 604 "pitparser.mly"
    ( Some _2 )
# 2975 "pitparser.ml"
               : 'optatident))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'gtermseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'optatident) in
    Obj.repr(
# 608 "pitparser.mly"
 ( PGFunApp (_1, _3, _5), parse_extent() )
# 2984 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 610 "pitparser.mly"
  ( PGIdent (_1), parse_extent() )
# 2991 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 612 "pitparser.mly"
        ( Param.has_integer := true; unfold_gint (gzero ()) _1 )
# 2998 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 614 "pitparser.mly"
        ( Param.has_integer := true; unfold_gint _1 _3 )
# 3006 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : int) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 616 "pitparser.mly"
        ( Param.has_integer := true; unfold_gint _3 _1 )
# 3014 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 618 "pitparser.mly"
        ( Param.has_integer := true; PGFunApp(("<=", parse_extent()), [_1; _3], None), parse_extent() )
# 3022 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 620 "pitparser.mly"
        ( Param.has_integer := true; PGFunApp((">=", parse_extent()), [_1; _3], None), parse_extent() )
# 3030 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 622 "pitparser.mly"
        ( Param.has_integer := true; PGFunApp(("<", parse_extent()), [_1; _3], None), parse_extent() )
# 3038 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 624 "pitparser.mly"
        ( Param.has_integer := true; PGFunApp((">", parse_extent()), [_1; _3], None), parse_extent() )
# 3046 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 6 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 4 : 'gtermseq) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : int) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'optatident) in
    Obj.repr(
# 626 "pitparser.mly"
        ( PGPhase(_1, _3, _6, _7), parse_extent() )
# 3056 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 4 : 'gterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : int) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'optatident) in
    Obj.repr(
# 628 "pitparser.mly"
        ( PGPhase(("table", parse_extent()), [_3], _6, _7), parse_extent() )
# 3065 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 630 "pitparser.mly"
        ( PGFunApp(("=", parse_extent()), [_1; _3], None), parse_extent() )
# 3073 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 632 "pitparser.mly"
        ( PGFunApp(("<>", parse_extent()), [_1; _3], None), parse_extent() )
# 3081 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'gterm) in
    Obj.repr(
# 634 "pitparser.mly"
        ( PGFunApp(("not", parse_extent()), [_3], None), parse_extent() )
# 3088 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 636 "pitparser.mly"
        ( PGFunApp(("||", parse_extent()), [_1; _3], None), parse_extent() )
# 3096 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 638 "pitparser.mly"
        ( PGFunApp(("&&", parse_extent()), [_1; _3], None), parse_extent() )
# 3104 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'gterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'gterm) in
    Obj.repr(
# 640 "pitparser.mly"
        ( PGFunApp(("choice", parse_extent()), [_3; _5], None), parse_extent() )
# 3112 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'gtermseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'optatident) in
    Obj.repr(
# 642 "pitparser.mly"
        ( PGFunApp(("event",parse_extent()), _3, _5), parse_extent() )
# 3120 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'gtermseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'optatident) in
    Obj.repr(
# 644 "pitparser.mly"
        ( PGFunApp(("inj-event",parse_extent()), _3, _5), parse_extent() )
# 3128 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'optatident) in
    Obj.repr(
# 646 "pitparser.mly"
        ( PGFunApp(("table",parse_extent()), [_3], _5), parse_extent() )
# 3136 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 648 "pitparser.mly"
        ( PGFunApp(("==>", parse_extent()), [_1;_3], None), parse_extent() )
# 3144 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'gtermseq) in
    Obj.repr(
# 650 "pitparser.mly"
 ( match _2 with
	  [t] -> t   (* Allow parentheses for priorities of infix operators;
			Tuples cannot have one element. *)
	| l -> PGTuple (l), parse_extent() )
# 3154 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'bindingseq) in
    Obj.repr(
# 655 "pitparser.mly"
        ( PGName (_2, _4), parse_extent() )
# 3162 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 657 "pitparser.mly"
        ( PGName (_2, []), parse_extent() )
# 3169 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 659 "pitparser.mly"
        ( PGLet(_2, _4, _6), parse_extent() )
# 3178 "pitparser.ml"
               : 'gterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'negtermseq) in
    Obj.repr(
# 663 "pitparser.mly"
 ( _1 :: _3 )
# 3186 "pitparser.ml"
               : 'negtermseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 665 "pitparser.mly"
 ( [_1] )
# 3193 "pitparser.ml"
               : 'negtermseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'negtermseq) in
    Obj.repr(
# 669 "pitparser.mly"
        ( _1 )
# 3200 "pitparser.ml"
               : 'gtermseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 671 "pitparser.mly"
        ( [] )
# 3206 "pitparser.ml"
               : 'gtermseq))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : int) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'nesbindingseq) in
    Obj.repr(
# 676 "pitparser.mly"
        ( (("!" ^ (string_of_int (_2)), parse_extent()), _4) :: _6 )
# 3215 "pitparser.ml"
               : 'nesbindingseq))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : int) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 678 "pitparser.mly"
        ( [(("!" ^ (string_of_int (_2)), parse_extent()), _4)] )
# 3223 "pitparser.ml"
               : 'nesbindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'gterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'nesbindingseq) in
    Obj.repr(
# 680 "pitparser.mly"
        ( (_1, _3) :: _5 )
# 3232 "pitparser.ml"
               : 'nesbindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gterm) in
    Obj.repr(
# 682 "pitparser.mly"
        ( [(_1, _3)] )
# 3240 "pitparser.ml"
               : 'nesbindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'nesbindingseq) in
    Obj.repr(
# 686 "pitparser.mly"
        ( _1 )
# 3247 "pitparser.ml"
               : 'bindingseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 688 "pitparser.mly"
        ( [] )
# 3253 "pitparser.ml"
               : 'bindingseq))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'gformat) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'tfnebindingseq) in
    Obj.repr(
# 694 "pitparser.mly"
        ( BFLet(_2, _4, _6) )
# 3262 "pitparser.ml"
               : 'tfnebindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'gformatseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'optphase) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'optint) in
    Obj.repr(
# 696 "pitparser.mly"
        ( BFNoUnif((_1,_3,_5), _6) )
# 3272 "pitparser.ml"
               : 'tfnebindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.ident) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'optint) in
    Obj.repr(
# 698 "pitparser.mly"
        ( BFNoUnif((_1,[],-1),_2) )
# 3280 "pitparser.ml"
               : 'tfnebindingseq))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'gformatseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'optphase) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'optint) in
    Obj.repr(
# 700 "pitparser.mly"
        ( BFNoUnif((("table", parse_extent()),_3,_5), _6) )
# 3289 "pitparser.ml"
               : 'tfnebindingseq))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'gformatseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'optphase) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'optint) in
    Obj.repr(
# 702 "pitparser.mly"
        ( BFNoUnif((("event", parse_extent()),_3,_5), _6) )
# 3298 "pitparser.ml"
               : 'tfnebindingseq))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 706 "pitparser.mly"
    ( _2 )
# 3305 "pitparser.ml"
               : 'optphase))
; (fun __caml_parser_env ->
    Obj.repr(
# 708 "pitparser.mly"
    ( -1 )
# 3311 "pitparser.ml"
               : 'optphase))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 712 "pitparser.mly"
    ( _2 )
# 3318 "pitparser.ml"
               : 'optint))
; (fun __caml_parser_env ->
    Obj.repr(
# 714 "pitparser.mly"
    ( -1 )
# 3324 "pitparser.ml"
               : 'optint))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'gformatseq) in
    Obj.repr(
# 718 "pitparser.mly"
 ( PFGFunApp (_1, _3), parse_extent() )
# 3332 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'gformat) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'gformat) in
    Obj.repr(
# 720 "pitparser.mly"
 ( PFGFunApp (("choice", parse_extent()), [_3; _5]), parse_extent() )
# 3340 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 722 "pitparser.mly"
 ( PFGIdent (_1), parse_extent() )
# 3347 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'gformatseq) in
    Obj.repr(
# 724 "pitparser.mly"
 ( match _2 with
	  [t] -> t   (* Allow parentheses for priorities of infix operators;
			Tuples cannot have one element. *)
	| l -> PFGTuple (_2), parse_extent() )
# 3357 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'fbindingseq) in
    Obj.repr(
# 729 "pitparser.mly"
        ( PFGName (_2, _4), parse_extent() )
# 3365 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 731 "pitparser.mly"
        ( PFGName (_2, []), parse_extent() )
# 3372 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 733 "pitparser.mly"
        ( PFGAny (_2), parse_extent() )
# 3379 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'gformat) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'gformat) in
    Obj.repr(
# 735 "pitparser.mly"
        ( PFGLet(_2, _4, _6), parse_extent() )
# 3388 "pitparser.ml"
               : 'gformat))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'gformat) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'negformatseq) in
    Obj.repr(
# 740 "pitparser.mly"
 ( _1 :: _3 )
# 3396 "pitparser.ml"
               : 'negformatseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'gformat) in
    Obj.repr(
# 742 "pitparser.mly"
 ( [_1] )
# 3403 "pitparser.ml"
               : 'negformatseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'negformatseq) in
    Obj.repr(
# 746 "pitparser.mly"
        ( _1 )
# 3410 "pitparser.ml"
               : 'gformatseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 748 "pitparser.mly"
        ( [] )
# 3416 "pitparser.ml"
               : 'gformatseq))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : int) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'gformat) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'fnesbindingseq) in
    Obj.repr(
# 753 "pitparser.mly"
        ( (("!" ^ (string_of_int (_2)), parse_extent()), _4) :: _6 )
# 3425 "pitparser.ml"
               : 'fnesbindingseq))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : int) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'gformat) in
    Obj.repr(
# 755 "pitparser.mly"
        ( [(("!" ^ (string_of_int (_2)), parse_extent()), _4)] )
# 3433 "pitparser.ml"
               : 'fnesbindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'gformat) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'fnesbindingseq) in
    Obj.repr(
# 757 "pitparser.mly"
        ( (_1, _3) :: _5 )
# 3442 "pitparser.ml"
               : 'fnesbindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'gformat) in
    Obj.repr(
# 759 "pitparser.mly"
        ( [(_1, _3)] )
# 3450 "pitparser.ml"
               : 'fnesbindingseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fnesbindingseq) in
    Obj.repr(
# 763 "pitparser.mly"
        ( _1 )
# 3457 "pitparser.ml"
               : 'fbindingseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 765 "pitparser.mly"
        ( [] )
# 3463 "pitparser.ml"
               : 'fbindingseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 771 "pitparser.mly"
    ( true )
# 3469 "pitparser.ml"
               : 'optorfail))
; (fun __caml_parser_env ->
    Obj.repr(
# 773 "pitparser.mly"
    ( false)
# 3475 "pitparser.ml"
               : 'optorfail))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'neidentseq) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'typeid) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'optorfail) in
    Obj.repr(
# 777 "pitparser.mly"
 ( List.map (fun x -> (x,_3,_4)) _1 )
# 3484 "pitparser.ml"
               : 'mayfailvartype))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'mayfailvartype) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'nemayfailvartypeseq) in
    Obj.repr(
# 781 "pitparser.mly"
 ( _1 @ _3 )
# 3492 "pitparser.ml"
               : 'nemayfailvartypeseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'mayfailvartype) in
    Obj.repr(
# 784 "pitparser.mly"
 ( _1 )
# 3499 "pitparser.ml"
               : 'nemayfailvartypeseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'nemayfailvartypeseq) in
    Obj.repr(
# 788 "pitparser.mly"
        ( _1 )
# 3506 "pitparser.ml"
               : 'mayfailvartypeseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 790 "pitparser.mly"
        ( [] )
# 3512 "pitparser.ml"
               : 'mayfailvartypeseq))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'nemayfailvartypeseq) in
    Obj.repr(
# 794 "pitparser.mly"
 ( _2 )
# 3519 "pitparser.ml"
               : 'forallmayfailvartype))
; (fun __caml_parser_env ->
    Obj.repr(
# 796 "pitparser.mly"
 ( [] )
# 3525 "pitparser.ml"
               : 'forallmayfailvartype))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : 'forallmayfailvartype) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.term_e) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.term_e) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'treducotherwise) in
    Obj.repr(
# 801 "pitparser.mly"
 ( (_2,_3,_5) :: _6 )
# 3535 "pitparser.ml"
               : 'treducotherwise))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'forallmayfailvartype) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 803 "pitparser.mly"
 ( [_2,_3,_5] )
# 3544 "pitparser.ml"
               : 'treducotherwise))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'forallmayfailvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.term_e) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.term_e) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'treducotherwise) in
    Obj.repr(
# 807 "pitparser.mly"
 ( (_1,_2,_4) :: _5 )
# 3554 "pitparser.ml"
               : 'treducmayfail))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'forallmayfailvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 809 "pitparser.mly"
 ( [_1,_2,_4] )
# 3563 "pitparser.ml"
               : 'treducmayfail))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'forallvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.term_e) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'treduc) in
    Obj.repr(
# 813 "pitparser.mly"
 ( (_1,_2,_4) :: _6 )
# 3573 "pitparser.ml"
               : 'treduc))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'forallvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 815 "pitparser.mly"
 ( [_1,_2,_4] )
# 3582 "pitparser.ml"
               : 'treduc))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'forallvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 821 "pitparser.mly"
    ( [(_1, _2)] )
# 3590 "pitparser.ml"
               : 'eqlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'forallvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'eqlist) in
    Obj.repr(
# 823 "pitparser.mly"
    ( (_1, _2)::_4 )
# 3599 "pitparser.ml"
               : 'eqlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 829 "pitparser.mly"
        ( PClause(_1,_3) )
# 3607 "pitparser.ml"
               : 'tclause))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 831 "pitparser.mly"
        ( PFact(_1) )
# 3614 "pitparser.ml"
               : 'tclause))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 833 "pitparser.mly"
        ( PEquiv(_1,_3,true) )
# 3622 "pitparser.ml"
               : 'tclause))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.term_e) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.term_e) in
    Obj.repr(
# 835 "pitparser.mly"
        ( PEquiv(_1,_3,false) )
# 3630 "pitparser.ml"
               : 'tclause))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'forallmayfailvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'tclause) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'tclauses) in
    Obj.repr(
# 839 "pitparser.mly"
 ( (_1,_2) :: _4 )
# 3639 "pitparser.ml"
               : 'tclauses))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'forallmayfailvartype) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'tclause) in
    Obj.repr(
# 841 "pitparser.mly"
 ( [_1,_2] )
# 3647 "pitparser.ml"
               : 'tclauses))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'progoptlist) in
    Obj.repr(
# 848 "pitparser.mly"
        ( _2 )
# 3654 "pitparser.ml"
               : 'programoptions))
; (fun __caml_parser_env ->
    Obj.repr(
# 850 "pitparser.mly"
        ( [] )
# 3660 "pitparser.ml"
               : 'programoptions))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'progopt) in
    Obj.repr(
# 854 "pitparser.mly"
        ( [_1] )
# 3667 "pitparser.ml"
               : 'progoptlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'progopt) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'progoptlist) in
    Obj.repr(
# 856 "pitparser.mly"
        ( _1 :: _3 )
# 3675 "pitparser.ml"
               : 'progoptlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 860 "pitparser.mly"
        ( ("Write", _1,_3) )
# 3683 "pitparser.ml"
               : 'progopt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 862 "pitparser.mly"
        ( ("Read", _1,_3) )
# 3691 "pitparser.ml"
               : 'progopt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'programoptions) in
    Obj.repr(
# 866 "pitparser.mly"
        ((_1,_2))
# 3699 "pitparser.ml"
               : 'progbegin))
; (fun __caml_parser_env ->
    Obj.repr(
# 870 "pitparser.mly"
        (true)
# 3705 "pitparser.ml"
               : 'progend))
; (fun __caml_parser_env ->
    Obj.repr(
# 872 "pitparser.mly"
        (false)
# 3711 "pitparser.ml"
               : 'progend))
; (fun __caml_parser_env ->
    Obj.repr(
# 878 "pitparser.mly"
    ( None )
# 3717 "pitparser.ml"
               : 'syncopt))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.ident) in
    Obj.repr(
# 880 "pitparser.mly"
    ( match _4, _5, _6 with
    | ("no",_), ("tag",_), ("prefix", _) -> Some ("",parse_extent())
    | ("tag",_), ("prefix",_), id -> Some id
    | _ -> raise Parsing.Parse_error )
# 3729 "pitparser.ml"
               : 'syncopt))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'progbegin) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 887 "pitparser.mly"
        ( _2 (* CryptoVerif implementation annotations ignored *) )
# 3737 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'tprocess) in
    Obj.repr(
# 889 "pitparser.mly"
 ( _2 )
# 3744 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.ident) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'syncopt) in
    Obj.repr(
# 891 "pitparser.mly"
        ( PLetDef (_1, [], _2), parse_extent() )
# 3752 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'ptermseq) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'syncopt) in
    Obj.repr(
# 893 "pitparser.mly"
        ( PLetDef (_1, _3, _5), parse_extent() )
# 3761 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 895 "pitparser.mly"
 ( PRepl _2, parse_extent() )
# 3768 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : Pitptree.ident) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 897 "pitparser.mly"
 ( (* For convergence with CryptoVerif, we allow an identifier (bound on the number of copies) after a replication; it is simply ignored in ProVerif. *)
          PRepl _5, parse_extent() )
# 3778 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 900 "pitparser.mly"
        ( (* For convergence with CryptoVerif, we allow "foreach i<=N do P"
             as a synonym for !P *)
          PRepl _6, parse_extent() )
# 3789 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 904 "pitparser.mly"
 ( let x = _1 in
	  if x = 0 then PNil, parse_extent() else
          input_error ("The only integer in a process is 0 for the nil process") (parse_extent()) )
# 3798 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    Obj.repr(
# 908 "pitparser.mly"
        ( (* For convergence with CryptoVerif, we allow yield instead of 0 *)
          PNil, parse_extent() )
# 3805 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'newarg) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'typeid) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 911 "pitparser.mly"
        ( PRestr(_2, _3, _5, _6), parse_extent() )
# 3815 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'typeid) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 913 "pitparser.mly"
        ( (* For convergence with CryptoVerif, we allow x <-R T
	     as a synonym for new x: T *)
          PRestr(_1, None, _3, _4), parse_extent() )
# 3826 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'pterm) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'tprocess) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'optelseprocess) in
    Obj.repr(
# 917 "pitparser.mly"
 ( PTest(_2,_4,_5), parse_extent() )
# 3835 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 5 : 'pterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 3 : 'tpattern) in
    let _7 = (Parsing.peek_val __caml_parser_env 1 : 'options) in
    let _8 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 919 "pitparser.mly"
 ( PInput(_3,_5,_8,_7), parse_extent() )
# 3845 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 5 : 'pterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 3 : 'pterm) in
    let _7 = (Parsing.peek_val __caml_parser_env 1 : 'progend) in
    let _8 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 921 "pitparser.mly"
 ( POutput(_3,_5,_8), parse_extent() )
# 3855 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'tpattern) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 923 "pitparser.mly"
 ( PLet(_2,_4,(PNil,parse_extent()),(PNil,parse_extent())), parse_extent() )
# 3863 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'tpattern) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'pterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'tprocess) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'optelseprocess) in
    Obj.repr(
# 925 "pitparser.mly"
 ( PLet(_2,_4,_6,_7), parse_extent() )
# 3873 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'basicpattern) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 927 "pitparser.mly"
        ( (* For convergence with CryptoVerif, we allow x[:T] <- M; P
	     as a synonym for let x[:T] = M in P *)
          PLet(_1,_3,_4,(PNil,parse_extent())), parse_extent() )
# 3884 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'options) in
    Obj.repr(
# 931 "pitparser.mly"
        ( PLetFilter(_2,_4,(PNil,parse_extent()),(PNil,parse_extent()),_5), parse_extent() )
# 3893 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 6 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 4 : 'pterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 3 : 'options) in
    let _7 = (Parsing.peek_val __caml_parser_env 1 : 'tprocess) in
    let _8 = (Parsing.peek_val __caml_parser_env 0 : 'optelseprocess) in
    Obj.repr(
# 933 "pitparser.mly"
        ( (* Approximating the else clause with a parallel composition
	     is not correct for trace reconstruction *)
          PLetFilter(_2,_4,_7,_8,_5), parse_extent() )
# 3906 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'ptermseq) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 937 "pitparser.mly"
        ( PInsert(_2, _4, _6), parse_extent() )
# 3915 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 7 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 5 : 'tpatternseq) in
    let _6 = (Parsing.peek_val __caml_parser_env 3 : 'optsuchthat) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : 'options) in
    let _8 = (Parsing.peek_val __caml_parser_env 1 : 'optinprocess) in
    let _9 = (Parsing.peek_val __caml_parser_env 0 : 'optelseprocess) in
    Obj.repr(
# 939 "pitparser.mly"
        ( PGet(_2, _4, _6, _8, _9,_7), parse_extent() )
# 3927 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'tprocess) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 941 "pitparser.mly"
 ( PPar(_1,_3), parse_extent() )
# 3935 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'ptermseq) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'newarg) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 943 "pitparser.mly"
        ( PEvent(_2, _4, _6, _7), parse_extent() )
# 3945 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'newarg) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 945 "pitparser.mly"
        ( PEvent(_2, [], _3, _4), parse_extent() )
# 3954 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : int) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 947 "pitparser.mly"
        ( if (_2) <= 0 then
	    input_error "Phases should be positive integers in processes" (parse_extent());
          PPhase(_2, _3), parse_extent() )
# 3964 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : int) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 951 "pitparser.mly"
        ( if (_2) <= 0 then
	    input_error "Sync numbers should be positive integers" (parse_extent());
          Param.has_barrier := true;
	  PBarrier(_2, None, _3), parse_extent() )
# 3975 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : int) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'opttprocess) in
    Obj.repr(
# 956 "pitparser.mly"
        ( if (_2) <= 0 then
	    input_error "Sync numbers should be positive integers" (parse_extent());
          Param.has_barrier := true;
	  PBarrier(_2, Some _4, _6), parse_extent() )
# 3987 "pitparser.ml"
               : 'tprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 963 "pitparser.mly"
        ( _2 )
# 3994 "pitparser.ml"
               : 'opttprocess))
; (fun __caml_parser_env ->
    Obj.repr(
# 965 "pitparser.mly"
        ( PNil, parse_extent() )
# 4000 "pitparser.ml"
               : 'opttprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 969 "pitparser.mly"
        ( _2 )
# 4007 "pitparser.ml"
               : 'optinprocess))
; (fun __caml_parser_env ->
    Obj.repr(
# 971 "pitparser.mly"
        ( PNil, parse_extent() )
# 4013 "pitparser.ml"
               : 'optinprocess))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'tprocess) in
    Obj.repr(
# 975 "pitparser.mly"
        ( _2 )
# 4020 "pitparser.ml"
               : 'optelseprocess))
; (fun __caml_parser_env ->
    Obj.repr(
# 977 "pitparser.mly"
        ( PNil, parse_extent() )
# 4026 "pitparser.ml"
               : 'optelseprocess))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 981 "pitparser.mly"
    ( PPatVar(_1,None) )
# 4033 "pitparser.ml"
               : 'basicpattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'typeid) in
    Obj.repr(
# 983 "pitparser.mly"
    ( PPatVar(_1, Some _3) )
# 4041 "pitparser.ml"
               : 'basicpattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'basicpattern) in
    Obj.repr(
# 987 "pitparser.mly"
    ( _1 )
# 4048 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'tpatternseq) in
    Obj.repr(
# 989 "pitparser.mly"
    ( match _2 with
	  [t] -> t   (* Allow parentheses for priorities of infix operators;
			Tuples cannot have one element. *)
	| l -> PPatTuple(_2) )
# 4058 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'tpatternseq) in
    Obj.repr(
# 994 "pitparser.mly"
    ( PPatFunApp(_1,_3) )
# 4066 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'tpattern) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'tpattern) in
    Obj.repr(
# 996 "pitparser.mly"
    ( Param.has_choice := true;
      PPatChoice(("choice", parse_extent()), [_3; _5], None) )
# 4075 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 5 : 'tpattern) in
    let _5 = (Parsing.peek_val __caml_parser_env 3 : 'tpattern) in
    let _8 = (Parsing.peek_val __caml_parser_env 0 : 'typeid) in
    Obj.repr(
# 999 "pitparser.mly"
    ( Param.has_choice := true;
      PPatChoice(("choice", parse_extent()), [_3; _5], Some _8) )
# 4085 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 1002 "pitparser.mly"
    ( Param.has_integer := true; unfold_pat_int (pat_zero()) _1 )
# 4092 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'tpattern) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 1004 "pitparser.mly"
    ( Param.has_integer := true; unfold_pat_int _1 _3 )
# 4100 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : int) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'tpattern) in
    Obj.repr(
# 1006 "pitparser.mly"
    ( Param.has_integer := true; unfold_pat_int _3 _1 )
# 4108 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1008 "pitparser.mly"
    ( PPatEqual(_2) )
# 4115 "pitparser.ml"
               : 'tpattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'tpattern) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'nepatternseq) in
    Obj.repr(
# 1012 "pitparser.mly"
    ( _1 :: _3 )
# 4123 "pitparser.ml"
               : 'nepatternseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'tpattern) in
    Obj.repr(
# 1014 "pitparser.mly"
    ( [_1] )
# 4130 "pitparser.ml"
               : 'nepatternseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'nepatternseq) in
    Obj.repr(
# 1018 "pitparser.mly"
    ( _1 )
# 4137 "pitparser.ml"
               : 'tpatternseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 1020 "pitparser.mly"
    ( [] )
# 4143 "pitparser.ml"
               : 'tpatternseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'ptermseq) in
    Obj.repr(
# 1026 "pitparser.mly"
 ( PPFunApp (_1, _3), parse_extent() )
# 4151 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'pterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    Obj.repr(
# 1028 "pitparser.mly"
        ( Param.has_choice := true;
	  PPFunApp(("choice", parse_extent()), [_3; _5]), parse_extent() )
# 4160 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 1031 "pitparser.mly"
 ( PPIdent (_1), parse_extent() )
# 4167 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 1033 "pitparser.mly"
        ( Param.has_integer := true; unfold_pint (pzero ()) _1 )
# 4174 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 1035 "pitparser.mly"
        ( Param.has_integer := true; unfold_pint_minus _1 _3 )
# 4182 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : int) in
    Obj.repr(
# 1037 "pitparser.mly"
        ( Param.has_integer := true; unfold_pint _1 _3 )
# 4190 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : int) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1039 "pitparser.mly"
        ( Param.has_integer := true; unfold_pint _3 _1 )
# 4198 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1041 "pitparser.mly"
        ( Param.has_integer := true; PPFunApp(("<", parse_extent()), [_1; _3]), parse_extent() )
# 4206 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1043 "pitparser.mly"
        ( Param.has_integer := true; PPFunApp((">", parse_extent()), [_1; _3]), parse_extent() )
# 4214 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1045 "pitparser.mly"
        ( Param.has_integer := true; PPFunApp(("<=", parse_extent()), [_1; _3]), parse_extent() )
# 4222 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1047 "pitparser.mly"
        ( Param.has_integer := true; PPFunApp((">=", parse_extent()), [_1; _3]), parse_extent() )
# 4230 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1049 "pitparser.mly"
        ( PPFunApp(("=", parse_extent()), [_1; _3]), parse_extent() )
# 4238 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1051 "pitparser.mly"
        ( PPFunApp(("<>", parse_extent()), [_1; _3]), parse_extent() )
# 4246 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    Obj.repr(
# 1053 "pitparser.mly"
        ( PPFunApp(("not", parse_extent()), [_3]), parse_extent() )
# 4253 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1055 "pitparser.mly"
        ( PPFunApp(("||", parse_extent()), [_1; _3]), parse_extent() )
# 4261 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1057 "pitparser.mly"
        ( PPFunApp(("&&", parse_extent()), [_1; _3]), parse_extent() )
# 4269 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 4 : 'newarg) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'typeid) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1059 "pitparser.mly"
        ( PPRestr(_2, _3, _5, _7), parse_extent() )
# 4279 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'typeid) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1061 "pitparser.mly"
        ( PPRestr(_1, None, _3, _5), parse_extent() )
# 4288 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'pterm) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'optelseterm) in
    Obj.repr(
# 1063 "pitparser.mly"
 ( PPTest(_2,_4,_5), parse_extent() )
# 4297 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'tpattern) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'pterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'optelseterm) in
    Obj.repr(
# 1065 "pitparser.mly"
 ( PPLet(_2,_4,_6,_7), parse_extent() )
# 4307 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'basicpattern) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1067 "pitparser.mly"
        ( PPLet(_1,_3,_5,None), parse_extent() )
# 4316 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : 'nevartype) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'pterm) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'optelseterm) in
    Obj.repr(
# 1069 "pitparser.mly"
        ( PPLetFilter(_2,_4,_6,_7), parse_extent() )
# 4326 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'optargs) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'newarg) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1071 "pitparser.mly"
        ( PPEvent(_2, _3, _4, _6), parse_extent() )
# 4336 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : 'ptermseq) in
    let _7 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1073 "pitparser.mly"
        ( PPInsert(_2,_4,_7), parse_extent() )
# 4345 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 8 : Pitptree.ident) in
    let _4 = (Parsing.peek_val __caml_parser_env 6 : 'tpatternseq) in
    let _6 = (Parsing.peek_val __caml_parser_env 4 : 'optsuchthat) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'options) in
    let _9 = (Parsing.peek_val __caml_parser_env 1 : 'pterm) in
    let _10 = (Parsing.peek_val __caml_parser_env 0 : 'optelseterm) in
    Obj.repr(
# 1075 "pitparser.mly"
        ( PPGet(_2,_4,_6,_9,_10,_7), parse_extent() )
# 4357 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'ptermseq) in
    Obj.repr(
# 1077 "pitparser.mly"
 ( match _2 with
	  [t] -> t   (* Allow parentheses for priorities of infix operators;
			Tuples cannot have one element. *)
	| l -> PPTuple (l), parse_extent() )
# 4367 "pitparser.ml"
               : 'pterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1084 "pitparser.mly"
    ( Some _2 )
# 4374 "pitparser.ml"
               : 'optelseterm))
; (fun __caml_parser_env ->
    Obj.repr(
# 1086 "pitparser.mly"
    ( None )
# 4380 "pitparser.ml"
               : 'optelseterm))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1090 "pitparser.mly"
    ( Some _2 )
# 4387 "pitparser.ml"
               : 'optsuchthat))
; (fun __caml_parser_env ->
    Obj.repr(
# 1092 "pitparser.mly"
    ( None )
# 4393 "pitparser.ml"
               : 'optsuchthat))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'ptermseq) in
    Obj.repr(
# 1096 "pitparser.mly"
    ( _2 )
# 4400 "pitparser.ml"
               : 'optargs))
; (fun __caml_parser_env ->
    Obj.repr(
# 1098 "pitparser.mly"
    ( [] )
# 4406 "pitparser.ml"
               : 'optargs))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'pterm) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'neptermseq) in
    Obj.repr(
# 1102 "pitparser.mly"
 ( _1 :: _3 )
# 4414 "pitparser.ml"
               : 'neptermseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'pterm) in
    Obj.repr(
# 1104 "pitparser.mly"
 ( [_1] )
# 4421 "pitparser.ml"
               : 'neptermseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'neptermseq) in
    Obj.repr(
# 1108 "pitparser.mly"
        ( _1 )
# 4428 "pitparser.ml"
               : 'ptermseq))
; (fun __caml_parser_env ->
    Obj.repr(
# 1110 "pitparser.mly"
        ( [] )
# 4434 "pitparser.ml"
               : 'ptermseq))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 1116 "pitparser.mly"
    ( [_1;_3] )
# 4442 "pitparser.ml"
               : 'onepermut))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'onepermut) in
    Obj.repr(
# 1118 "pitparser.mly"
    ( _1 :: _3 )
# 4450 "pitparser.ml"
               : 'onepermut))
; (fun __caml_parser_env ->
    Obj.repr(
# 1122 "pitparser.mly"
    ( [] )
# 4456 "pitparser.ml"
               : Pitptree.ident list list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'onepermut) in
    Obj.repr(
# 1124 "pitparser.mly"
    ( [_1] )
# 4463 "pitparser.ml"
               : Pitptree.ident list list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'onepermut) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident list list) in
    Obj.repr(
# 1126 "pitparser.mly"
    ( _1 :: _3 )
# 4471 "pitparser.ml"
               : Pitptree.ident list list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : Pitptree.ident) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident list) in
    Obj.repr(
# 1134 "pitparser.mly"
    ( _1 :: _3 )
# 4479 "pitparser.ml"
               : Pitptree.ident list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Pitptree.ident) in
    Obj.repr(
# 1136 "pitparser.mly"
    ( [_1] )
# 4486 "pitparser.ml"
               : Pitptree.ident list))
(* Entry all *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry lib *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry permut *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry order *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry term *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let all (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Pitptree.tdecl list * Pitptree.tprocess_e * Pitptree.tprocess_e option)
let lib (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 2 lexfun lexbuf : Pitptree.tdecl list)
let permut (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 3 lexfun lexbuf : Pitptree.ident list list)
let order (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 4 lexfun lexbuf : Pitptree.ident list)
let term (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 5 lexfun lexbuf : Pitptree.term_e)
